######################################################################################################################
# Preference Reversal Study Control File  	                                           			   Collins and James #
######################################################################################################################
#																													 #
# 	Sean M. Collins							Duncan James															 #
# 	Assistant Professor of Economics		Associate Professor of Economics										 #
# 	Fordham University						Fordham University														 #
#																													 #
# 	113 W 60th St Rm 924					441 East Fordham Road													 #
# 	New York NY 10023-7414					Bronx, NY 10458															 #
# 	(646) 312-8270							(718) 817-4053															 #
# 	scollins15@fordham.edu					dujames@fordham.edu														 #
#																													 #
######################################################################################################################
# 																													 #
# Notes																												 #
#																													 #
# 1. You must have first process the data with the main.r file before attempting to use these functions, because     #
#    they assume that you have generated the appropriate derivative files, e.g. subjectX.csv files.					 #
#																													 #
# 2. a. The treatments are named differently in this code than in the paper. The translations are:					 #
#	    Up = CE, Down = PE, Left = PL, Right = PR																	 #
#    b. The order of the games in the paper (P1,P2,...,P5) is reversed from the ordering in the data; that is  		 #
#    	(P5,P4,...,P1). Compare the output to the paper to better understand this.									 #
#	 c. "Chained dominance violations" in the paper are termed "monotonicity violations" here.						 #
#																													 #
# 3. It will be helpful to read Blavastkyy (2009, 2012) and our Online Appendix before proceeding.					 #
#																													 #
# 4. Bootstrapping can take a LONG time with (current) desktop-class hardware.										 #
#																													 #
######################################################################################################################
# Clear workspace and load libraries.																				 #
######################################################################################################################

# Set working directory and remove all objects. (Remember to set your working directory!)
# setwd("");
# rm(list=ls(all=TRUE))

######################################################################################################################
# Declare functions for utility, the psi transform, and the distribution and density functions.						 #
######################################################################################################################

# Define VNM utility function and its partial derivatives (with respect to payoff and probability).
    u   <- function(X, pX, r) { return(pX*(X^(1-r))/(1-r)); }	# CRRA expected utility with zero downstate.
  u.x   <- function(X, pX, r) { return(pX*X^(-r)); } 			# Derivative of CRRA w.r.t. the lottery payoff, X.
  u.p   <- function(X, r)     { return(X^(1-r)/(1-r)); } 		# Derivative of CRRA w.r.t. the lottery probability, pX.

# Define curvature function psi and its derivative as per Blavatskyy.
    psi <- function(X,a) { return(X^a); } 					# Power psi.
  psi.d <- function(X,a) { return(a*X^(a-1)); }				# Derivative of power psi.
#   psi <- function(X,a) { return(exp(a*X)); } 				# Exponential psi.
# psi.d <- function(X,a) { return(a*exp(a*X)); }			# Derivative of exponential psi.

# Define the cumulative distribution function as per Blavatskyy (assuming downstate is zero always, as in our case). 
F <- function(A,B,pA,pB,a,r)
{
	L      <- pmin(A,B)   # apply(cbind(A,B),1,min);		# Calculate the upstate outcome of the upper lower bound lottery.
	pL     <- pmin(pA,pB) # apply(cbind(pA,pB),1,min);		# Calculate the upstate prob of the upper lower bound lottery.
	
	uL     <- u(L,pL,r);									# Calculate the utility of the upper lower bound lottery.
	psiA   <- psi(u(A,pA,r)-uL,a);							# Calculate the transformed utility of lottery A minus L.
	psiAB  <- psiA+psi(u(B,pB,r)-uL,a);						# Calculate the sum transformed utilities of lotteries.
	
	return(psiA/psiAB);										# Return value of the distribution function.
}

# Define the probability density function for the probability equivalent, optionally filtering out zero densities.
# This function will automatically type experiment-generated data, but BC may be specified for non-certainty cash equivalents.
f <- function(A,B,pA,pB,a,r,filter=FALSE)
{
	if (filter==TRUE)										# If filter is enabled, only calculate PDF for nonzero densities.
	{
		h    <- F(A,B,pA,pB,a,r)%%1>0;						# Determine where the the density is nonzero.
		z    <- rep(0,length(h));							# Create a vector of zeros to return as the density.
		z[h] <- f(A[h],B[h],pA[h],pB[h],a,r);				# Calculate and assign non-zero densities.
		return(z);											# Return densities.
	}

	L      <- pmin(A,B)										# Calculate the upstate outcome of the upper lower bound lottery.
	pL     <- pmin(pA,pB)									# Calculate the upstate probability of the upper lower bound lottery.

# 	if (any(BC))											# If treatment is specified, use the specified treatment.
# 	{
# 		CE <- (BC==-1)										# Set indicator for certainty equivalence.
# 		PE <- (BC==-2|BC==-3)								# Set indicator for probability equivalence.
# 		PA <- (BC==-4)										# Set indicator for probability equivalence where pL=PB (PL).
# 	}
# 	else													# If treatment is not specified, assign based on parameters.
# 	{
		CE <- (pA==1);										# Set indicator for certainty equivalence.
		PE <- !CE;											# Set indicator for probability equivalence.
		PA <- PE&(pL==pA);									# Set indicator for probability equivalence where pL=PA (PE & PR).
		PB <- PE&(pL==pB);									# Set indicator for probability equivalence where pL=PB (PL).
#	}

	uL     <- u(L,pL,r)									    # The upper lower bound lottery for PE(pA<1) is pAºB.
	uA     <- u(A,pA,r)-uL									# Calculate the utility of lottery A minus L.
	uB     <- u(B,pB,r)-uL									# Calculate the utility of lottery B minus L.
	psiA   <- psi(uA,a)										# Calculate the transformed utility of lottery A minus L.
	psiAB  <- psiA+psi(uB,a)								# Calculate the sum transformed utilities of lotteries.
	
	uL.d       <- rep(0,length(A))							# Create space to store the marginal utility of the upper lower bound lottery.
	uA.d       <- rep(0,length(A))							# Create space to store the marginal utility of A minus L.

	uL.d[CE]   <- u.x(L[CE],pL[CE],r)						# If CE, calculate the marginal utility of the upper lower bound lottery.
	uL.d[PE]   <- u.p(L[PE],r)								# If PE, calculate the marginal utility of the upper lower bound lottery.
	uA.d[CE]   <- u.x(A[CE],pA[CE],r)-uL.d[CE]				# If CE, the MU of A minus L is equal to the difference of MUA & MUL.
	uA.d[PA]   <- u.p(A[PA],r)-uL.d[PA]						# If PE and pL=pA, the MU of A minus L is equal to the difference of MUA & MUL.
	uA.d[PB]   <- uL.d[PB]									# If PE and pL=pB, the marginal utility of A minus L is equal to the MU of L.
	
	psiA.d     <- uA.d*psi.d(uA,a)							# Calculate the derivative of the transformed utility of A minus L.
	psiB.d     <- rep(0,length(A))							# If PE and pL=pB, the derivative of transformed utility of B minus L is zero.
	psiB.d[CE] <- -uL.d[CE]*psi.d(uB[CE],a)					# If CE, calculate the derivative of transformed utility of B minus L.
	psiB.d[PA] <- -uL.d[PA]*psi.d(uB[PA],a)					# If pL=pA, calculate the derivative of transformed utility of B minus L.
	
	return((psiA.d/psiAB)-((psiA.d+psiB.d)*psiA)/(psiAB^2)) # Return the value of the density function.
}

# Define the truncated probability density function for the probability equivalent – this is used for PL.
f.tr <- function(pA,pB,A,B,a,r,filter=FALSE) { return(f(A,B,pA,pB,a,r,filter)/F(A,B,1,pB,a,r)); }

# Define  functions to draw random samples from the inverse cumulative distribution function.
Fi.x <- function(A,B,pA,pB,a,r,z)  { return(F(A,B,pA,pB,a,r)-z) }	# Return the payoff-first CDF minus a random variable to find F(z)=z.
Fi.p <- function(pA,pB,A,B,a,r,z)  { return(F(A,B,pA,pB,a,r)-z) }	# Return the probability-first CDF minus a random variable to find F(z)=z.
sampleF.x <- function(B,pA,pB,a,r)									# Generates sample certainty equivalents using the CDF.
{
	y <- 1:length(pA);
	for (i in y) { y[i] = uniroot(Fi.x,c(0,B[i]),B[i],pA[i],pB[i],a,r,runif(1))$root; }
	return(y);
}
sampleF.p <- function(A,B,pB,a,r)									# Generates sample probability equivalents (including PE, PL, & PR) using the CDF.
{																	# Note that in PL the CDF does not reach 1 at PA; thus the truncated draw.
	y <- 1:length(A);
	for (i in y) { y[i] <- uniroot(Fi.p,c(0,1),pB[i],A[i],B[i],a,r,runif(1,0,F(A[i],B[i],1,pB[i],a,r)))$root; }
	return(y);
}

# Define functions that return the probability that the CE or PE for A is greater than for B.
# Note the Riemann–Stieltjes integral ∫f(x)g'(x)dx=∫f(x)dg(x) where g(·) is differentiable and continuous.
Fd.x <- function(x,A,B,pA,pB,a,r)               { return(F(x,A,1,pA,a,r)*f(x,B,1,pB,a,r)); }   # The CE is x.
Fd.p <- function(q,A,B,pA,pB,qA,qB,a,r)         { return(F(qA,A,q,pA,a,r)*f(qB,B,q,pB,a,r)); } # The PE is q, qX is alt. payoff, & pX is alt . prob.
Pr.x <- function(A,B,pA,pB,a,r,lb=0,ub=B)       { return(integrate(Vectorize(Fd.x),lb,ub,A,B,pA,pB,a,r)$value); }       # The upper bound is B.
Pr.p <- function(A,B,pA,pB,qA,qB,a,r,lb=0,ub=1) { return(integrate(Vectorize(Fd.p),lb,ub,A,B,pA,pB,qA,qB,a,r)$value); } # The bounds are defined.

# Define functions that return the probability that the CE for A is greater than for B using change-of-variable as a check.
Fi.g <- function(x,A,B,pA,pB,a,r)    { return(F(uniroot(Fi.x,c(0,B),B,1,pB,a,r,x)$root,A,1,pA,a,r)); }
Pr.g <- function(A,B,pA,pB,a,r)      { return(integrate(Vectorize(Fi.g),0,F(A,B,1,pB,a,r),A,B,pA,pB,a,r)$value); }

# Define functions to allow numerical differentiation as a check.
F.pn <- function(pA,pB,A,B,a,r) { return(F(A,B,pA,pB,a,r)) }
f.xn <- function(A,B,pA,pB,a,r) { return(grad(F,A,method="Richardson", method.args=list(),B,pA,pB,a,r)); }
f.pn <- function(A,B,pA,pB,a,r) { return(grad(F.pn,pA,method="Richardson", method.args=list(),pB,A,B,a,r)); }

######################################################################################################################
# Declare functions to calculate log-likelihood																		 #
######################################################################################################################

# Calculate the (joint) log-likelihood function for the Blavatskyy model.
F.ll <- function(p,A,B,pA,pB,BC)
{
#	# Bounce unacceptable values of alpha and rho.
#	if (p[1]>-1e-4)     		 { p[1] <- 1e-4;   }			# Bounce alpha=0 since f(x)=0, F(x)=0.5, and alpha<0 since it is problematic.
#	if (p[2]>1-1e-4&p[2]<1+1e-4) { p[2] <- 1+1e-4; }			# Bounce rho=1, since f(x) and F(x) are undefined.

#	# Reject out-of-bounds values of alpha and rho.
#	if (p[1]<0)  { return(Inf); }								# Reject alpha=0 since f(x)=0, F(x)=0.5, and alpha<0 since it is problematic.
#	if (p[2]==1) { return(Inf); }								# Reject rho=1, since f(x) and F(x) are undefined.

	# Create indicator variables for treatment.
	H  <- (BC>=0); 												# Create indicator variable for binary choice.
	h  <- !H;													# Create indicator for BDM elicitations (CE, PE, PL, & PR)

	# Calculate the value of the CDF or PDF.
	Pr   <- F(A[H],B[H],pA[H],pB[H],p[1],p[2]);					# Calculate values of the distribution function for BC.
	pr   <- f(A[h],B[h],pA[h],pB[h],p[1],p[2]);					# Calculate values of the density function for BDM. 
	
	# Calculate and filter the log-likelihood.
	lnll <- sum(log(pr),BC[H]*log(Pr)+(1-BC[H])*log(1-Pr));		# Calculate the joint log-likelihood function.
	lnll[is.na(lnll)]<--Inf;									# Reject NaN and NA values of the log-likelihood function.
	return(-lnll);								 				# Return inverse log-likelihood (for minimization).	
}

# Calculate the (joint) log-likelihood function for a given rho with the Blavatskyy model.
A.ll <- function(a,A,B,pA,pB,BC,r) { return(F.ll(c(a,r),A,B,pA,pB,BC)); }

# Use a grid search to maximize the log-likelihood function for the Blavatskyy model.
grid <- function(lower,upper,fn,A,B,pA,pB,BC,N=101)
{
	# Create vector of parameters over which to search the log-likelihood function.
	alpha <- seq(lower[1],upper[1],len=N);						# Declare search vector for alpha.
	rho   <- seq(lower[2],upper[2],len=N);						# Declare search vector for rho.
	lnll  <- matrix(NaN,N,N);									# Create space to store log-likelihood values.

	# Calculate the log-likelihood of each {alpha,rho} pair.
	for (i in 1:N)
	{
		for (j in 1:N)
		{
			p <- c(alpha[i],rho[j]);                       		# Extract {alpha, rho} for the lnll row and column.
			lnll[i,j] <- fn(p,A,B,pA,pB,BC) 					# Calculate the log-likelihood of {alpha,rho}.
		}
	}

	# Find and print the alpha and rho that maximize the log-likelihood function.
	lli <- which(lnll==min(lnll),arr.ind=TRUE)					# Get indices of rows and columns that minimize lnll.
	return(c(alpha[lli[1]],rho[lli[2]],min(lnll),NA));			# Return alpha, rho, minimized lnll, and NA.
}

# Calculate the (joint) log-likelihood function for logistic CRRA estimation from Binary Choice.
R.ll <- function(r,A,B,pA,pB,BC)
{
	
	H    <- (BC>=0) 											# Filter the data so only binary choice is used.
	Pr   <- plogis(u(A[H],pA[H],r)-u(B[H],pB[H],r));			# Calculate the log of the difference of utilities transformed by the logistic C.D.F.
	lnll <- sum(BC*log(Pr)+(1-BC)*log(1-Pr));					# Calculate the log-likelihood function.
	lnll[is.na(lnll)]<--Inf;									# Reject NaN and NA values of the log-likelihood function.
	return(-lnll);									 			# Return inverse log-likelihood (for minimization).	
}

######################################################################################################################
# Declare functions to fetch processed data for estimation.															 #
######################################################################################################################

# Return the experimental parameters as a data frame.
param.B <- function() { return(read.csv(paste("parameters.csv",sep=""))); }

# Return raw data for a selected subject as a data frame.
disp.B <- function(s) { return(read.csv(paste("subject_",s,".csv",sep=""))) }

# Extract and format data for treatment(s) "t" from subject(s) "s."
fetch.B <- function(s=NA,t,omit=TRUE,...)
{
	# Create a data frame to store extracted data.
	z <- data.frame()
	
	# Declare function to bound binary choice probabilities that fall out of the range [0,1].
	# (This function is unnecessary given the parameterization and filters we've employed.)
	# b <- function(p) { return(pmax(pmin(p,rep(1,length(p))),rep(0,length(p)))) }
	
	for (i in s)
	{
		# Get raw data for selected subject and parameters.
		if (!is.na(i)) { x <- disp.B(i); }
		else           { x <- sample.B(...); }
		p <- param.B();

		if (any(t=="CE"|t=="HCE"))									# Extract Certainty Equivalence (PE) choice data.
		{
			A    <- c(x$HCE[1:5],x$HCE[11:15]);						# Get outcomes for lottery A (the deterministic lottery).
			B    <- c(x$HoutcomeA[1:5],x$HoutcomeA[11:15]);			# Get outcomes for lottery B (the "alternate" lottery).
			pA   <- rep(1,10);										# Get probabilities for lottery A.
			pB   <- c(x$HprobA[1:5],x$HprobA[11:15])/100;					# Get probabilities for lottery B. 
			BC   <- rep(-1,10);										# Set indicator for binary choice as -1 for CE.
			y <- as.data.frame(cbind(A,B,pA,pB,BC)); 				# Move extracted data to a data frame for filtering.
			if (omit) { y <- y[((y$B-y$A>1e-2)&(y$A>1e-2)),] }		# Omit observations with prob of 1 or 0.
			else
			{
				y$A[y$B-y$A<1e-2] <- y$B[y$B-y$A<1e-2]-.01;			# Push observations w/ prob of 1 to 1-e.
				y$A[y$A<1e-2] <- .01;								# Push observations w/ prob of 0 to 0+e.
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="CE"|t=="HCX"))									# Extract Certainty Equivalence (PE) choice data.
		{
			A    <- x$HCE[6:10];									# Get outcomes for lottery A (the deterministic lottery).
			B    <- x$HoutcomeA[6:10];								# Get outcomes for lottery B (the "alternate" lottery).
			pA   <- rep(1,5);										# Get probabilities for lottery A.
			pB   <- x$HprobA[6:10]/100;								# Get probabilities for lottery B. 
			BC   <- rep(-1,5);										# Set indicator for binary choice as -1 for CE.
			y <- as.data.frame(cbind(A,B,pA,pB,BC))[-2,];			# Row 7 is a copy of row 2 and is removed.
			if (omit) { y <- y[((y$B-y$A>1e-2)&(y$A>1e-2)),] }		# Omit observations with prob of 1 or 0.
			else
			{
				y$A[y$B-y$A<1e-2] <- y$B[y$B-y$A<1e-2]-.01;			# Push observations w/ prob of 1 to 1-e.
				y$A[y$A<1e-2] <- .01;								# Push observations w/ prob of 0 to 0+e.
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="PE"|t=="HPE"))									# Extract Probability Equivalence (PE) choice data.
		{
			A    <- c(x$HoutcomeA[1:5],x$HoutcomeA[11:15]);			# Get outcomes for lottery A (the lottery set by BDM).
			B    <- c(x$Hupfront[1:5],x$Hupfront[11:15]);			# Get outcomes for lottery B (the deterministic lottery).
			pA   <- c(x$HPE[1:5],x$HPE[11:15])/100;					# Get probabilities for lottery A.
			pB   <- rep(1,10);										# Get probabilities for lottery B.
			BC   <- rep(-2,10);										# Set indicator for binary choice as -2 for PE.
			y <- as.data.frame(cbind(A,B,pA,pB,BC)) 				# Move extracted data to a data frame for filtering.
			if (omit) { y <- y[((y$pA<1-1e-2)&(y$pA>+1e-2)),] }		# Omit observations with prob of 1 or 0.
			else
			{
				y$pA[y$pA>1-1e-2] <- 1-.01;							# Push observations w/ prob of 1 to 1-e.
				y$pA[y$pA<1e-2] 	 <- .01;						# Push observations w/ prob of 0 to 0+e.
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}	
		if (any(t=="PL"|t=="HPL"))									# Extract Project Left (PL) choice data.
		{
			# Extract PL choice data.
			A    <- c(x$HoutcomeA[6:10],x$HoutcomeA[16:20]);		# Get outcomes for lottery A (the "initial" lottery).
			B    <- rep(80,10);										# Get outcomes for lottery B (the reference lottery).
			pA   <- c(x$HPL[6:10],x$HPL[16:20])/100;				# Get probabilities for lottery A.
			pB   <- c(x$HrefprobA[6:10],x$HrefprobA[16:20])/100;	# Get probabilities for lottery B.
			BC   <- rep(-3,10);										# Set indicator for binary choice as -3 for PL.
			y <- as.data.frame(cbind(A,B,pA,pB,BC))					# Move extracted data to a data frame for filtering.
			if (omit)
			{
				y <- y[((y$pA<1-1e-2)&(y$pA>+1e-2)),]				# Omit observations with prob of 1 or 0.
				y <- y[y$pA>=y$pB,]									# Omit observations with prob of 1 or 0 (pA<pB).
			}
			else
			{
				y$pA[y$pA>1-1e-2] <- 1-.01;							# Push observations w/ prob of 1 to 1-e.
			#	y$pA[y$pA<1e-2]   <- .01;							# Push observations w/ prob of 0 to 0+e (unnecessary b/c pB>0).
				y$pA[y$pA<y$pB]   <- y$pB[y$pA<y$pB]+.01			# Push observations w/ prob of 0 to 0+e (pA<pB).
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="PR"|t=="HPR"))									# Extract Project Right (PR) choice data.
		{
			A    <- rep(80,10);										# Get outcomes for lottery A (the reference lottery).
			B    <- c(x$HoutcomeA[6:10],x$HoutcomeA[16:20]);		# Get outcomes for lottery B (the "alternate" lottery).
			pA   <- c(x$HPR[6:10],x$HPR[16:20])/100;				# Get probabilities for lottery A.
			pB   <- c(x$HprobA[6:10],x$HprobA[16:20])/100;			# Get probabilities for lottery B.
			BC   <- rep(-4,10);										# Set indicator for binary choice as -4 for PR.
			y <- as.data.frame(cbind(A,B,pA,pB,BC));				# Move extracted data to a data frame for filtering.
			if (omit)
			{
				y <- y[((y$pA<1-1e-2)&(y$pA>+1e-2)),]				# Omit observations with prob of 1 or 0.
				y <- y[y$pA<=y$pB,]									# Omit observations with prob of 1 or 0 (pA>pB).
			}
			else
			{
			#	y$pA[y$pA>1-1e-2] <- 1-.01;							# Push observations w/ prob of 1 to 1-e (unnecessary b/c pB<1).
				y$pA[y$pA<1e-2]   <- .01;							# Push observations w/ prob of 0 to 0+e.
				y$pA[y$pA>y$pB]   <- y$pB[y$pA>y$pB]-.01			# Push observations w/ prob of 1 to 1-e (pA>pB).
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="CE"|t=="RCE"))									# Extract Time-Consistency Certainty Equivalence (RCE) data.
		{
			A    <- x$RCE[c(2,4,12,14)];							# Get outcomes for lottery A.
			B    <- x$HoutcomeA[c(2,4,12,14)];						# Get outcomes for lottery B.
			pA   <- rep(1,4);										# Get probabilities for lottery A.
			pB   <- x$HprobA[c(2,4,12,14)]/100;						# Get probabilities for lottery B.
			BC   <- rep(-1,4);										# Set indicator for binary choice as -1 for CE.
			y <- as.data.frame(cbind(A,B,pA,pB,BC))					# Move extracted data to a data frame for filtering.
			if (omit) { y <- y[((y$B-y$A>1e-2)&(y$A>1e-2)),] }		# Omit observations with prob of 1 or 0.
			else
			{
				y$A[y$B-y$A<1e-2] <- y$B[y$B-y$A<1e-2]-.01;			# Push observations w/ prob of 1 to 1-e.
				y$A[y$A<1e-2] <- .01;								# Push observations w/ prob of 0 to 0+e.
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="CE"|t=="RCX"))									# Extract Time-Consistency Certainty Equivalence (RCE) data.
		{
			A    <- x$RCE[9];										# Get outcome for lottery A.
			B    <- x$HoutcomeA[9];									# Get outcome for lottery B.
			pA   <- rep(1,1);										# Get probability for lottery A.
			pB   <- x$HprobA[9]/100;								# Get probability for lottery B.
			BC   <- rep(-1,1);										# Set indicator for binary choice as -1 for CE.
			y <- as.data.frame(cbind(A,B,pA,pB,BC))					# Move extracted data to a data frame for filtering.
			if (omit) { y <- y[((y$B-y$A>1e-2)&(y$A>1e-2)),] }		# Omit observations with prob of 1 or 0.
			else
			{
				y$A[y$B-y$A<1e-2] <- y$B[y$B-y$A<1e-2]-.01;			# Push observations w/ prob of 1 to 1-e.
				y$A[y$A<1e-2] <- .01;								# Push observations w/ prob of 0 to 0+e.
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="PE"|t=="RPE"))									# Extract Time-Consistency Probability Equivalence (RPE) data.
		{
			A    <- x$HoutcomeA[c(2,4,12,14)]						# Get outcomes for lottery A.
			B    <- x$Hupfront[c(2,4,12,14)]						# Get outcomes for lottery B.
			pA   <- x$HPE[c(2,4,12,14)]/100;						# Get probabilities for lottery A.
			pB   <- rep(1,4);										# Get probabilities for lottery B.
			BC   <- rep(-2,4);										# Set indicator for binary choice as -2 for PE.
			y <- as.data.frame(cbind(A,B,pA,pB,BC)) 				# Move extracted data to a data frame for filtering.
			if (omit) { y <- y[((y$pA<1-1e-2)&(y$pA>+1e-2)),] }		# Omit observations with prob of 1 or 0.
			else
			{
				y$pA[y$pA>1-1e-2] <- 1-.01;							# Push observations w/ prob of 1 to 1-e.
				y$pA[y$pA<1e-2] 	 <- .01;						# Push observations w/ prob of 0 to 0+e.
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="PL"|t=="RPL"))									# Extract Time-Consistency Project Left (RPL) data.
		{
			# Extract PL choice data.
			A    <- x$HoutcomeA[c(7,9,17,19)]						# Get outcomes for lottery A.
			B    <- rep(80,4)										# Get outcomes for lottery B.
			pA   <- x$HPL[c(7,9,17,19)]/100;						# Get probabilities for lottery A.
			pB   <- x$HrefprobA[c(7,9,17,19)]/100					# Get probabilities for lottery B.
			BC   <- rep(-3,4);										# Set indicator for binary choice as -3 for PL.
			y <- as.data.frame(cbind(A,B,pA,pB,BC))					# Move extracted data to a data frame for filtering.
			if (omit)
			{
				y <- y[((y$pA<1-1e-2)&(y$pA>+1e-2)),]				# Omit observations with prob of 1 or 0.
				y <- y[y$pA>=y$pB,]									# Omit observations with prob of 1 or 0 (pA<pB).
			}
			else
			{
				y$pA[y$pA>1-1e-2] <- 1-.01;							# Push observations w/ prob of 1 to 1-e.
			#	y$pA[y$pA<1e-2]   <- .01;							# Push observations w/ prob of 0 to 0+e (unnecessary b/c pB>0).
				y$pA[y$pA<y$pB]   <- y$pB[y$pA<y$pB]+.01			# Push observations w/ prob of 0 to 0+e (pA<pB).
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="PR"|t=="HPR"))									# Extract Project Right (PR) choice data.
		{
			A    <- rep(80,4);										# Get outcomes for lottery A.
			B    <- x$HoutcomeA[c(7,9,17,19)];						# Get outcomes for lottery B.
			pA   <- x$HPR[c(7,9,17,19)]/100;						# Get probabilities for lottery A.
			pB   <- x$HprobA[c(7,9,17,19)]/100;						# Get probabilities for lottery B.
			BC   <- rep(-4,4);										# Set indicator for binary choice as -4 for PR.
			y <- as.data.frame(cbind(A,B,pA,pB,BC));				# Move extracted data to a data frame for filtering.
			if (omit)
			{
				y <- y[((y$pA<1-1e-2)&(y$pA>+1e-2)),]				# Omit observations with prob of 1 or 0.
				y <- y[y$pA<=y$pB,]									# Omit observations with prob of 1 or 0 (pA>pB).
			}
			else
			{
			#	y$pA[y$pA>1-1e-2] <- 1-.01;							# Push observations w/ prob of 1 to 1-e (unnecessary b/c pB<1).
				y$pA[y$pA<1e-2]   <- .01;							# Push observations w/ prob of 0 to 0+e.
				y$pA[y$pA>y$pB]   <- y$pB[y$pA>y$pB]-.01			# Push observations w/ prob of 1 to 1-e (pA>pB).
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}		
		if (any(t=="BC"|t=="HBC"|t=="BCE"))							# Extract binary choice data for certainty equivalent (CE).
		{
			A      <- x$HoutcomeA[1:5];								# Get outcomes for lottery A.
			B      <- x$HoutcomeA[11:15];							# Get outcomes for lottery B.
			pA     <- x$HprobA[1:5]/100;							# Get probabilities for lottery A (pre endogeneity adjustment).
			pB     <- x$HprobA[11:15]/100;							# Get probabilities for lottery B (pre endogeneity adjustment).
			BC     <- x$BCE[1:5]									# Get subject choices as binary (1 for A, 0 for B).
			H      <- (x$HCE[1:5]>=x$HCE[11:15]);					# Create indicator for whether lottery A had a greater CE than B.
			pA[H]  <- pA[H]+(p$SBCE[1:5]/100)[H];					# If lottery A was preferred in CE, sweeten lottery B.
			pB[H]  <- pB[H]-(p$BBCE[11:15]/100)[H];					# If lottery A was preferred in CE, bitter lottery B.
			pA[!H] <- pA[!H]-(p$BBCE[1:5]/100)[!H];					# If lottery B was preferred in CE, bitter lottery A.
			pB[!H] <- pB[!H]+(p$SBCE[11:15]/100)[!H];				# If lottery B was preferred in CE, sweeten lottery B.
			z <- as.data.frame(rbind(z,cbind(A,B,pA,pB,BC)))		# Add extracted data to data frame.
		}
		if (any(t=="BC"|t=="BCX"))									# Extract binary choice data for certainty equivalent across (CE).
		{
			A      <- x$HoutcomeA[6:10];							# Get outcomes for lottery A.
			B      <- rep(x$HoutcomeA[12],5);						# Get outcomes for lottery B.
			pA     <- x$HprobA[6:10]/100;							# Get probabilities for lottery A (pre endogeneity adjustment).
			pB     <- rep(x$HprobA[12],5)/100;						# Get probabilities for lottery B (pre endogeneity adjustment).
			BC     <- x$BCE[6:10]									# Get subject choices as binary (1 for A, 0 for B).
			H      <- (x$HCE[6:10]>=rep(x$HCE[12],5));				# Create indicator for whether lottery A had a greater CE than B.
			pA[H]  <- pA[H]+(p$SBCE[6:10]/100)[H];					# If lottery A was preferred in CE, sweeten lottery B.
			pB[H]  <- pB[H]-(p$BBCE[6:10]/100)[H];					# If lottery A was preferred in CE, bitter lottery B.
			pA[!H] <- pA[!H]-(p$BBCE[6:10]/100)[!H];				# If lottery B was preferred in CE, bitter lottery A.
			pB[!H] <- pB[!H]+(p$SBCE[6:10]/100)[!H];				# If lottery B was preferred in CE, sweeten lottery B.
			y <- as.data.frame(cbind(A,B,pA,pB,BC))[-5,];			# Row 5 is ommitted because the outcome is deterministic. 
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}		
		if (any(t=="BC"|t=="HBC"|t=="BPE"))							# Extract binary choice data for probability equivalent (PE).
		{
			A      <- x$HoutcomeA[1:5];								# Get outcomes for lottery A.
			B      <- x$HoutcomeA[11:15];							# Get outcomes for lottery B.
			pA     <- x$HPE[1:5]/100;								# Get probabilities for lottery A (pre endogeneity adjustment).
			pB     <- x$HPE[11:15]/100;								# Get probabilities for lottery B (pre endogeneity adjustment).
			BC     <- x$BPE[1:5]									# Get subject choices as binary (1 for A, 0 for B).
			H      <- (x$Hupfront[1:5]>=x$Hupfront[11:15]);			# Create indicator for whether lottery A had greater upfront than B.
			pA[H]  <- pA[H]+(p$SBPE[1:5]/100)[H];					# If lottery A was preferred in PE, sweeten lottery B.
			pB[H]  <- pB[H]-(p$BBPE[11:15]/100)[H];					# If lottery A was preferred in PE, bitter lottery B.
			pA[!H] <- pA[!H]-(p$BBPE[1:5]/100)[!H];					# If lottery B was preferred in PE, bitter lottery A.
			pB[!H] <- pB[!H]+(p$SBPE[11:15]/100)[!H];				# If lottery B was preferred in PE, sweeten lottery B.
			y <- as.data.frame(cbind(A,B,pA,pB,BC));
			if (omit)												# [((y$A>y$B)&(y$pA<y$pB))|((y$B>y$A)&(y$pA>y$pB))]
			{														# (The above code would be used generally, but A<B always.)
				y <- y[y$pA>y$pB,];									# Omit observations with prob of 1 or 0 (pA>pB since A<B).
				y <- y[y$pA>0&y$pA<1&y$pB>0&y$pB<1,];				# Omit observations with prob of 1 or 0.
			}						
			else
			{
				y$pA[y$pA<y$pB] <- y$pB[y$pA<y$pB]+1e-2;			# Push observations w/ prob of 0 to 0+e (pA<pB with A<B).
				y$pA[y$pA<=0]   <- 1e-2;							# Push observations in lottery A w/ prob of 0 to 0+e.
				y$pA[y$pA>=1]   <- 1-1e-2;							# Push observations in lottery A w/ prob of 1 to 1-e.
				y$pB[y$pB<=0]   <- 1e-2;							# Push observations in lottery B w/ prob of 0 to 0+e.
				y$pB[y$pB>=1]   <- 1-1e-2;							# Push observations in lottery B w/ prob of 1 to 1-e
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="BC"|t=="BPL"))									# Extract binary choice data for leftward projection (PL).
		{
			A      <- x$HoutcomeA[6:10];							# Get outcomes for lottery A.
			B      <- x$HoutcomeA[16:20];							# Get outcomes for lottery B.
			pA     <- x$HPL[6:10]/100;								# Get probabilities for lottery A (pre endogeneity adjustment).
			pB     <- x$HPL[16:20]/100;								# Get probabilities for lottery B (pre endogeneity adjustment).
			BC     <- x$BPL[6:10]									# Get subject choices as binary (1 for A, 0 for B).
			H      <- (x$HrefprobA[6:10]>=x$HrefprobA[16:20]);		# Create indicator for whether lottery A had a greater RP than B.
			pA[H]  <- pA[H]+(p$SBPL[6:10]/100)[H];					# If lottery A was preferred in PL, sweeten lottery B.
			pB[H]  <- pB[H]-(p$BBPL[16:20]/100)[H];					# If lottery A was preferred in PL, bitter lottery B.
			pA[!H] <- pA[!H]-(p$BBPL[6:10]/100)[!H];				# If lottery B was preferred in PL, bitter lottery A.
			pB[!H] <- pB[!H]+(p$SBPL[16:20]/100)[!H];				# If lottery B was preferred in PL, sweeten lottery B.
			y <- as.data.frame(cbind(A,B,pA,pB,BC)); 
			if (omit)
			{
				y <- y[y$pA>y$pB,];									# Omit observations with prob of 1 or 0 (pA>pB since A<B).
				y <- y[y$pA>0&y$pA<1&y$pB>0&y$pB<1,];				# Omit observations with prob of 1 or 0.
			}						
			else
			{
				y$pA[y$pA<y$pB] <- y$pB[y$pA<y$pB]+1e-2;			# Push observations w/ prob of 0 to 0+e (pA<pB with A<B).
				y$pA[y$pA<=0]   <- 1e-2;							# Push observations in lottery A w/ prob of 0 to 0+e.
				y$pA[y$pA>=1]   <- 1-1e-2;							# Push observations in lottery A w/ prob of 1 to 1-e.
				y$pB[y$pB<=0]   <- 1e-2;							# Push observations in lottery B w/ prob of 0 to 0+e.
				y$pB[y$pB>=1]   <- 1-1e-2;							# Push observations in lottery B w/ prob of 1 to 1-e
			}
			z <- as.data.frame(rbind(z,y))							# Add extracted data to data frame.
		}
		if (any(t=="BC"|t=="BPR"))									# Extract binary choice data for rightward projection (PR).
		{
			A      <- x$HoutcomeA[6:10];							# Get outcomes for lottery A.
			B      <- x$HoutcomeA[16:20];							# Get outcomes for lottery B.
			pA     <- x$HprobA[6:10]/100;							# Get probabilities for lottery A (pre endogeneity adjustment).
			pB     <- x$HprobA[16:20]/100;							# Get probabilities for lottery B (pre endogeneity adjustment).
			BC     <- x$BPR[6:10]									# Get subject choices as binary (1 for A, 0 for B).
			H      <- (x$HPR[6:10]>=x$HPR[16:20]);					# Create indicator for whether lottery A had a greater (PR)PE than B.
			pA[H]  <- pA[H]+(p$SBPR[6:10]/100)[H];					# If lottery A was preferred in PR, sweeten lottery B.
			pB[H]  <- pB[H]-(p$BBPR[16:20]/100)[H];					# If lottery A was preferred in PR, bitter lottery B.
			pA[!H] <- pA[!H]-(p$BBPR[6:10]/100)[!H];				# If lottery B was preferred in PR, bitter lottery A.
			pB[!H] <- pB[!H]+(p$SBPR[16:20]/100)[!H];				# If lottery B was preferred in PR, sweeten lottery B.
			z <- as.data.frame(rbind(z,cbind(A,B,pA,pB,BC)))		# Add extracted data to data frame.
		}
	}
	return(z)														# Return the final data frame.
}

# Generate sample data for a subject with a given parameter set using the Blavatskyy model.
sample.B <- function(a=NA,r=0,endogenize=TRUE)
{
	# Fetch relevant experimental parameters and create space for data.
	p <- param.B();
	x <- p[,1:4];
	x[c("HCE","HPE","HPL","HPR","BCE","BPE","BPL","BPR","RCE","RPE","RPL","RPR")] <- NA

	# Create space to store endogenization adjustments.
	PCE <- rep(0,22);
	PPE <- rep(0,22);
	PPL <- rep(0,22);
	PPR <- rep(0,22);
	
	# Declare function to bound probabilities that fall out of the range [0,100].
	b <- function(p) { return(pmax(pmin(p,rep(100,length(p))),rep(0,length(p)))) }

	# Generate deterministic BDM responses based on CRRA if the parameter alpha is given an NA value.
	if (is.na(a))
	{
		# Generate responses. (In CE, row 7 is copied to row 2, but the answer is deterministic here.)
		x$HCE[1:15]          <- round((x$HprobA/100)^(1/(1-r))*x$HoutcomeA,2)[1:15];
		x$HPE[c(1:5,11:15)]  <- round(b((x$Hupfront/x$HoutcomeA)^(1-r)*100),0)[c(1:5,11:15)];
		x$HPL[c(6:10,16:20)] <- round(b(x$HrefprobA*(80/x$HoutcomeA)^(1-r)),0)[c(6:10,16:20)];
		x$HPR[c(6:10,16:20)] <- round(b(x$HprobA*(x$HoutcomeA/80)^(1-r)),0)[c(6:10,16:20)];
		
		# Give the same response to consistency checks as to the initial questions.
		x$RCE[c(2,4,7,9,12,14)] <- x$HCE[c(2,4,7,9,12,14)];
		x$RPE[c(2,4,12,14)]     <- x$HPE[c(2,4,12,14)];
		x$RPL[c(7,9,17,19)] 	<- x$HPL[c(7,9,17,19)];
		x$RPR[c(7,9,17,19)] 	<- x$HPR[c(7,9,17,19)];
	}
	# Randomly generate BDM responses based on the Blavatskyy model if both parameters are supplied.
	else
	{
		# Randomly generate responses from Blavatskyy cumulative distribution function.
		x$HCE[1:15]          <- round(sampleF.x(x$HoutcomeA[1:15],rep(1,15),x$HprobA[1:15]/100,a,r),2);
		x$HPE[c(1:5,11:15)]  <- round(sampleF.p(x$HoutcomeA[c(1:5,11:15)],x$Hupfront[c(1:5,11:15)],rep(1,10),a,r)*100,0);
		x$HPL[c(6:10,16:20)] <- round(sampleF.p(x$HoutcomeA[c(6:10,16:20)],rep(80,10),x$HrefprobA[c(6:10,16:20)]/100,a,r)*100,0);
		x$HPR[c(6:10,16:20)] <- round(sampleF.p(rep(80,10),x$HoutcomeA[c(6:10,16:20)],x$HprobA[c(6:10,16:20)]/100,a,r)*100,0);
		x$HCE[7]             <- x$HCE[2] # The experiment copies row 2 over row 7, which is omitted.	
		
		# Give new responses to consistency checks.
		x$RCE[c(2,4,9,12,14)] <- round(sampleF.x(x$HoutcomeA[c(2,4,9,12,14)],rep(1,5),x$HprobA[c(2,4,9,12,14)]/100,a,r),2);
		x$RPE[c(2,4,12,14)]	  <- round(sampleF.p(x$HoutcomeA[c(2,4,12,14)],x$Hupfront[c(2,4,12,14)],rep(1,4),a,r)*100,0);
		x$RPL[c(7,9,17,19)]   <- round(sampleF.p(x$HoutcomeA[c(7,9,17,19)],rep(80,4),x$HrefprobA[c(7,9,17,19)]/100,a,r)*100,0);
		x$RPR[c(7,9,17,19)]   <- round(sampleF.p(rep(80,4),x$HoutcomeA[c(7,9,17,19)],x$HprobA[c(7,9,17,19)]/100,a,r)*100,0);
		x$RCE[7]              <- x$RCE[2]  # The experiment copies row 2 over row 7, which is omitted.
	}
	
	# Generate responses to consistency checks, which are always deterministic.
	x$HCE[21:22] <- c(x$HoutcomeA[21],0);
	x$HPE[21:22] <- c(x$HprobA[21],0);
	
	# If endogenization is triggered (as default), endoegenize choices for binary choice.
	if (endogenize)
	{
		# Create vectors of endogeneity adjustments.
		PCE[1:5]   <- as.numeric(x$HCE[1:5]>=x$HCE[11:15])*p$SBCE[1:5]-as.numeric(x$HCE[1:5]<x$HCE[11:15])*p$BBCE[1:5];
		PCE[11:15] <- as.numeric(x$HCE[1:5]<x$HCE[11:15])*p$SBCE[11:15]-as.numeric(x$HCE[1:5]>=x$HCE[11:15])*p$BBCE[11:15];
		PPE[1:5]   <- as.numeric(x$Hupfront[1:5]>=x$Hupfront[11:15])*p$SBPE[1:5]-as.numeric(x$Hupfront[1:5]<x$Hupfront[11:15])*p$BBPE[1:5];
		PPE[11:15] <- as.numeric(x$Hupfront[1:5]<x$Hupfront[11:15])*p$SBPE[11:15]-as.numeric(x$Hupfront[1:5]>=x$Hupfront[11:15])*p$BBPE[11:15];
		PPL[6:10]  <- as.numeric(x$HrefprobA[6:10]>=x$HrefprobA[16:20])*p$SBPL[6:10]-as.numeric(x$HrefprobA[6:10]<x$HrefprobA[16:20])*p$BBPL[6:10];
		PPL[16:20] <- as.numeric(x$HrefprobA[6:10]<x$HrefprobA[16:20])*p$SBPL[16:20]-as.numeric(x$HrefprobA[6:10]>=x$HrefprobA[16:20])*p$BBPL[16:20];	
		PPR[6:10]  <- as.numeric(x$HPR[6:10]>=x$HPR[16:20])*p$SBPR[6:10]-as.numeric(x$HPR[6:10]<x$HPR[16:20])*p$BBPR[6:10];
		PPR[16:20] <- as.numeric(x$HPR[6:10]<x$HPR[16:20])*p$SBPR[16:20]-as.numeric(x$HPR[6:10]>=x$HPR[16:20])*p$BBPR[16:20];		
		PCE[6:10]  <- as.numeric(x$HCE[6:10]>=rep(x$HCE[12],5))*p$SBCE[1:5]-as.numeric(x$HCE[6:10]<rep(x$HCE[12],5))*p$BBCE[1:5];
		PCE[16:20] <- as.numeric(x$HCE[6:10]<rep(x$HCE[12],5))*p$SBCE[1:5]-as.numeric(x$HCE[6:10]>=rep(x$HCE[12],5))*p$BBCE[1:5];
	}
	
	# Generate deterministic binary choice responses based on CRRA if the parameter alpha is given an NA value.
	if (is.na(a))
	{
		# Fill in binary choice responses.
		x$BCE[1:5]  <- as.numeric(u(x$HoutcomeA[1:5],b(x$HprobA[1:5]+PCE[1:5])/100,r)>=u(x$HoutcomeA[11:15],b(x$HprobA[11:15]+PCE[11:15])/100,r));
		x$BPE[1:5]  <- as.numeric(u(x$HoutcomeA[1:5],b(x$HPE[1:5]+PPE[1:5])/100,r)>=u(x$HoutcomeA[11:15],b(x$HPE[11:15]+PPE[11:15])/100,r));
		x$BPL[6:10] <- as.numeric(u(x$HoutcomeA[6:10],b(x$HPL[6:10]+PPL[6:10])/100,r)>=u(x$HoutcomeA[16:20],b(x$HPL[16:20]+PPL[16:20])/100,r));
		x$BPR[6:10] <- as.numeric(u(x$HoutcomeA[6:10],b(x$HprobA[6:10]+PPR[6:10])/100,r)>=u(x$HoutcomeA[16:20],b(x$HprobA[16:20]+PPR[16:20])/100,r));
		x$BCE[6:10] <- as.numeric(u(x$HoutcomeA[6:10],b(x$HprobA[6:10]+PCE[6:10])/100,r)>=u(rep(x$HoutcomeA[12],5),b(rep(x$HprobA[12],5)+PCE[16:20])/100,r));
	}
	# Randomly generate binary choice responses based on the Blavatskyy model if both parameters are supplied.
	else
	{
		# Randomly fill in binary choice responses from Blavatskyy cumulative distribution function.
		x$BCE[1:5]  <- as.numeric(F(x$HoutcomeA[1:5],x$HoutcomeA[11:15],b(x$HprobA[1:5]+PCE[1:5])/100,b(x$HprobA[11:15]+PCE[11:15])/100,a,r)>runif(5));
		x$BPE[1:5]  <- as.numeric(F(x$HoutcomeA[1:5],x$HoutcomeA[11:15],b(x$HPE[1:5]+PPE[1:5])/100,b(x$HPE[11:15]+PPE[11:15])/100,a,r)>runif(5));
		x$BPL[6:10] <- as.numeric(F(x$HoutcomeA[6:10],x$HoutcomeA[16:20],b(x$HPL[6:10]+PPL[6:10])/100,b(x$HPL[16:20]+PPL[16:20])/100,a,r)>runif(5));
		x$BPR[6:10] <- as.numeric(F(x$HoutcomeA[6:10],x$HoutcomeA[16:20],b(x$HprobA[6:10]+PPR[6:10])/100,b(x$HprobA[16:20]+PPR[16:20])/100,a,r)>runif(5));
		x$BCE[6:10] <- as.numeric(F(x$HoutcomeA[6:10],rep(x$HoutcomeA[12],5),b(x$HprobA[6:10]+PCE[6:10])/100,b(rep(x$HprobA[12],5)+PCE[16:20])/100,a,r)>runif(5));		
	}
	
	# Fill in the lower half of the binary choice vector with opposite responses.
	x$BCE[11:15] <- as.numeric(!x$BCE[1:5]);
	x$BPE[11:15] <- as.numeric(!x$BPE[1:5]);
	x$BPL[16:20] <- as.numeric(!x$BPL[6:10]);
	x$BPR[16:20] <- as.numeric(!x$BPR[6:10]);

	# Return the data frame of generated data.
	return(x);
}

# Generate a random sample and write to subject s<=0 for testing purposes.
generate.B <- function(s=0,a=1,r=0) { if (all(s<=0)) { write.csv(sample.B(a,r),paste("subject_",s,".csv",sep=""),row.names=FALSE); } }

######################################################################################################################
# Declare functions to estimate parameters of Blavatskyy model via maximum likelihood								 #
######################################################################################################################

# Calculate a CRRA rho for supplied data to generate starter values (parameter guesses) for estimation.
# If used for multiple treatments, Will return an INVALID weighted estimate to be used as a starter for MLE.
# If used for PE or CE alone, it will return an OLS-derived CRRA rho estimate. For BC, it returns the logistic choice CRRA estimate.
# These values are ony for starting points; they should not be used in analysis or otherwise interpretted.
start.B <- function(z)
{
	# Create indicator variables for treatments (note PL and PR fall under PE).
	BC <- (z$BC>=0);
	CE <- (z$BC==-1);
	PE <- (z$BC<=-2);

	# Create space to store variables and estimates of rho.
	y <- vector();
	x <- vector();
	r <- rep(0,3);

	if (any(CE))											# Estimate CE rho via OLS.
	{
		y[CE] <- log(z$A[CE]/z$B[CE])						# Declare CE independent variable.
		x[CE] <- log(z$pB[CE]/z$pA[CE])						# Declare PE dependent variable.
		r[1]  <- lm(y[CE]~0+x[CE])$coefficients[[1]];		# Estimate CE beta.
		r[1]  <- (r[1]-1)/r[1];								# Calculate rho=(beta-1)/beta.
	}
	if (any(PE))											# Estimate PE rho via OLS.
	{
		y[PE] <- log(z$pA[PE]/z$pB[PE])						# Declare PE independent variable.
		x[PE] <- log(z$B[PE]/z$A[PE])						# Declare PE dependent variable.
		r[2]  <- 1-lm(y[PE]~0+x[PE])$coefficients[[1]];		# Estimate and calculate PE rho=(1-beta).
	}
	if (any(BC))											# Estimate BC rho via MLE.
	{
		# r[3] <- suppressWarnings(optimize(R.ll,c(-1,1),z$A[BC],z$B[BC],z$pA[BC],z$pB[BC],z$BC[BC]))$minimum;
		BC.mle <- suppressWarnings(nlm(R.ll,.5,z$A[BC],z$B[BC],z$pA[BC],z$pB[BC],z$BC[BC],iterlim=1000000));
		if (BC.mle$code<=2) { r[3] <- BC.mle$estimate; }	# If MLE estimate returned no error, report it.
		else { r[3] <- NA; }								# Otherwise, discard estimate.
	}
	
	# Calculate rho as a weighted average of estimated rhos.
	r <- mean(CE)*r[1]+mean(PE)*r[2]+mean(BC)*r[3];
	
	# Calculate an alpha corresponding to the estimated rho.
	A.mle <- suppressWarnings(nlm(A.ll,1-1e-2,z$A,z$B,z$pA,z$pB,z$BC,r,iterlim=1000000))
	if (A.mle$code<=2) { a <- A.mle$estimate; }				# If MLE estimate returned no error, report it.
	else { a <- NA; }										# Otherwise, discard estimate.

	# Return estimate (or the weighted average of estimates, if multiple types of data were entered).
	return(c(a,r));
}

# Use MLE to estimate {alpha,rho} for subject s. (Can combine multiple subjects, although we don't.) Returns {alpha, rho, lnll, d} where d is convergence parameter.
# Note that nlm doesn't allow bounds and tends to stop on local minima rather than global, so optim() is being applied in this robust version.
mle.B <- function(z,lb=c(1e-4,-2),ub=c(2,.9),p=c(.75,.2),s=0,i=1,ll=TRUE,use.nlm=TRUE)
{
	# Determine how much information to return based on switch.
	if (ll) { h=1:4; } else { h=1:2; }

	# If there are not enough observations, report error.
	if (dim(z)[1]<=1)
	{
		if (s)  { print(paste("Subject ",s,", Nesting Level ",i,": Insufficient number of observations; skipping estimation.",sep="")); }
		else    { return(rep(NA,4)[h]); } 
	}
	else if (s&i==1) { print(paste("Subject ",s,", Nesting Level ",	i,": Beginning MLE estimation.",sep="")); }
	
	# First, try unconstrained non-linear minimization using a Newton-type algorithm and guesses based on CRRA.
	if (i==1&use.nlm)
	{
		mle <- suppressWarnings(try(nlm(F.ll,start.B(z),z$A,z$B,z$pA,z$pB,z$BC),silent=TRUE));
		if (substr(mle[1],1,5)!="Error")
		{
			# Fewer than one iteration is an indication that the routine was stuck.
			if (mle$iterations>1) 
			{
				if (s) { print(paste("Subject ",s,", Nesting Level ",i,": Sucessful estimation with nlm.",sep=""));	}
				return(c(mle$estimate[1],mle$estimate[2],mle$minimum,mle$code)[h]);
			}
		}
	}
	else if (s) { print(paste("Subject ",s,", Nesting Level ",i,": Unsucessful at estimation with nlm. Switching to L-BFGS-B.",sep="")); }
	
	# Set optimization method and function. (Recommended:"L-BFGS-B" for two dimensions, "Brent" for one dimension.)
	Method <- "L-BFGS-B";
	Fn     <- F.ll;
	
	# Use built-in optimization routine to estimate alpha and rho and Byrd et. al. (1995) with box constraints.
	y <- try(optim(p,method=Method,lower=lb,upper=ub,fn=Fn,A=z$A,B=z$B,pA=z$pA,pB=z$pB,BC=z$BC),silent=TRUE);

	# If the built-in routine returns an error, change the starting values and try again (given the low computational cost).
	if (substr(y[1],1,5)=="Error"&s) { print(paste("Subject ",s,", Nesting Level ",	i,": Changing starting values and reattempting optimization.",sep="")); }
	if (substr(y[1],1,5)=="Error"){ y <- try(optim(rowSums(cbind(p,lb))/2,method=Method,lower=lb,upper=ub,fn=Fn,A=z$A,B=z$B,pA=z$pA,pB=z$pB,BC=z$BC),silent=TRUE); }
	if (substr(y[1],1,5)=="Error"){ y <- try(optim(rowSums(cbind(p,ub))/2,method=Method,lower=lb,upper=ub,fn=Fn,A=z$A,B=z$B,pA=z$pA,pB=z$pB,BC=z$BC),silent=TRUE); }

	# If the built-in routine still fails and standard non-linear minimization is disabled, use the default method of Nelder and Mead (1965).
	if (substr(y[1],1,5)=="Error"&!use.nlm)
	{
		if (s) { print(paste("Subject ",s,", Nesting Level ",	i,": New starting values failed. Switching to Nelder and Mead (1965).",sep="")); }
		y <- try(optim(p,fn=Fn,A=z$A,B=z$B,pA=z$pA,pB=z$pB,BC=z$BC),silent=TRUE);
	}

	# If the built-in routine STILL fails, use a grid search to estimate alpha and rho.
	if (substr(y[1],1,5)=="Error"&s) { print(paste("Subject ",s,", Nesting Level ",	i,": New starting values failed. Switching to grid search for optimization.",sep="")); }
	if (substr(y[1],1,5)=="Error") { output <- grid(lower=lb,upper=ub,fn=Fn,A=z$A,B=z$B,pA=z$pA,pB=z$pB,BC=z$BC); } #|y$convergence>1e-4
	else { output <- c(y$par[1],y$par[2],y$value,y$convergence) }	
	
	# If the boundary is hit, run the routine again with new bounds.
	if ((((output[1]<=lb[1])|(output[1]>=ub[1]))|((output[2]<=ub[2])|(output[2]>=ub[2])))&s)
	{
		print(paste("Subject ",s,", Nesting Level ",i,": Parameters on boundary; reestimating new bounds.",sep=""));
	}
	# if    (output[1]<=lb[1]) { update <- mle.B(z,lb=c(2*lb[1],lb[2]),ub=ub,p,s,i=i+1); }#p=output[1:2]); } # Very low alphas cause problems.
	if  	(output[1]>=ub[1]) { update <- mle.B(z,lb=lb,ub=c(2*ub[1],ub[2]),p,s,i=i+1); }#p=output[1:2]); }
	else if (output[2]<=lb[2]) { update <- mle.B(z,lb=c(lb[1],2*lb[2]),ub=ub,p,s,i=i+1); }#p=output[1:2]); }
	else if (output[2]>=ub[2]) { update <- mle.B(z,lb=lb,ub=c(ub[1],2*ub[2]),p,s,i=i+1); }#p=output[1:2]); }
	else if (ll)			   { return(output);      }
	else                       { return(output[1:2]); }
	
	# If output was updated due to a bound hit, test to see if the value of the log-likelihood function is greater in the new estimate.
	if   (update[3]<output[3])
	{
		if (s)  { print(paste("Subject ",s,", Nesting Level ",	i,": Bound update was successful: LL ",round(update[3],digits=2)," < ",round(output[3],digits=2),sep="")); }
		if (ll) { return(update);      }
		else    { return(update[1:2]); }
	}
	else
	{
		if (s)  { print(paste("Subject ",s,", Nesting Level ",	i,": Bound update was unsuccessful: LL ",round(update[3],digits=2)," > ",round(output[3],digits=2),sep="")); }
		if (ll) { return(output);      }
		else    { return(output[1:2]); }
	}
}

# Use MLE to estimate separate {alpha,rho}'s for subject(s). Returns data frame of {alpha, rho, lnll, d} where d is convergence parameter in optim(), NA for grid search.
# This will also generate random samples of {obs} MLE parameters from some true {a,r} when called using the form: mle.A(t,rep(NA,obs),a=1,r=0)
mle.A <- function(t=c("CE","PE","BC"),s=1:60,R=NA,reversal=FALSE,monotonicity=FALSE,omit=TRUE,silent=TRUE,...)
{
	# Create a data frame to store outcomes of MLE by subject.
	y <- as.data.frame(t(as.data.frame(t(matrix(NA,length(s),4)),row.names=c("alpha","rho","lnLL","d"))),row.names=as.character(1:length(s))); 
	if (!is.na(R)) 	  { y[c("p.symmetric","p.equaltail","p.uppertail","p.lowertail")] <- NA }
	if (reversal)  	  { m <- data.frame(); }
	if (monotonicity) { n <- data.frame(); }
	
	# Do MLE estimations for every subject and store them in a data frame.
	for (i in 1:length(s))
	{
		y[i,] <- mle.B(fetch.B(s=s[i],t=t,omit=omit,...),s=(!silent)*i);
		if (!is.na(R)) # If R is not NA, conduct a bootstrap to determine p-values.
		{
			p     <- boot.B(a.hat=y$alpha[i],r.hat=y$rho[i],r.null=0,R=R,t=t,...);
			y[i,] <- cbind(y[i,1:4],p$rho[1],p$rho[2],p$rho[3],p$rho[4]);
		}
		if (reversal)     { m <- rbind(m,reversal.B(s=s[i],a=y$alpha[i],r=y$rho[i])); }
		if (monotonicity) { n <- rbind(n,monotonicity.B(s=s[i],a=y$alpha[i],r=y$rho[i])); }
	}
	if (!reversal&!monotonicity)   { return(y); } # Return MLE estimates as a data frame.
	else
	{
		if (reversal&monotonicity) { return(list(y,m,n)); } # Return a list consisting of MLE estimates and and reversal probabilities as data frames.
		else if (reversal)		   { return(list(y,m));   } # Return a list consisting of MLE estimates and and cross-institution reversal probabilities as data frames.
		else if (monotonicity)	   { return(list(y,n));   } # Return a list consisting of MLE estimates and and intra-institution reversal probabilities as data frames.
	} 																	
}

######################################################################################################################
# Declare functions for bootstrapping errors of MLE estimates of the Blavatskyy model.								 #
######################################################################################################################

# Calculate the p-values for bootstrapped {alpha,rho} estimates.
boot.B <- function(a.hat,r.hat,a.null=NA,r.null=NA,R=1,t,plot=FALSE,desc=FALSE)
{
	# Simulate the distribution of {alpha,rho} under the null hypothesis. Note that because the parameters are jointly estimated, they are
	# also jointly estimated here; the DGP must mirror that used in the MLE estimation.

	# If there no nulls are given, use the function to test that the DGP produces {alpha,rho} estimates that are asymptotically unbiased.
	if (all(is.na(c(r.null,a.null))))
	{
		a.null <- a.hat;
		r.null <- r.hat;
	}
	
	# If there at least one null is given, generate data under the null under the (joint) hypothesis.
	if (all(!is.na(c(r.null,a.null)))) { y <- mle.A(t=t,s=rep(NA,R),a=a.null,r=r.null); } # Simulate the distribution of {alpha,rho} estimates under the joint null hypothesis.
	else if (!is.na(r.null))           { y <- mle.A(t=t,s=rep(NA,R),a=a.hat,r=r.null);  } # Simulate the distribution of {alpha,rho} estimates under the null hypothesis for rho.
	else if (!is.na(a.null))           { y <- mle.A(t=t,s=rep(NA,R),a=a.null,r=r.hat);  } # Simulate the distribution of {alpha,rho} estimates under the null hypothesis for alpha.
	
	# Calculate and return the p-value by determining how often the estimates are generated by chance.
	# Here we obtain p-values for both {alpha,rho} as a check since the computational cost is low.
	p.uppertail <- c(sum(y$alpha-a.null>a.hat-a.null)/R,sum(y$rho-r.null>r.hat-r.null)/R);						# Calculate the upper-tail p-value.
	p.lowertail <- c(sum(y$alpha-a.null<a.hat-a.null)/R,sum(y$rho-r.null<r.hat-r.null)/R);						# Calculate the lower-tail p-value.
	p.equaltail <- c(2*min(p.lowertail[1],p.uppertail[1]),2*min(p.lowertail[2],p.uppertail[2]));				# Calculate the equal-tail p-value.
	p.symmetric <- c(sum(abs(y$alpha-a.null)>abs(a.hat-a.null))/R,sum(abs(y$rho-r.null)>abs(r.hat-r.null))/R);	# Calculate the symmetric p-value.
	
	# Calculate descriptive statistics from the bootstrap.
	y.mean      <- c(mean(y$alpha),mean(y$rho));
	y.sd        <- c(sd(y$alpha),sd(y$rho));
	
	# Combine calculate p-values as a data frame.
	p <- rbind(p.symmetric,p.equaltail,p.uppertail,p.lowertail);
	p <- as.data.frame(t(as.data.frame(t(p),row.names=c("alpha","rho"))));
	if (desc)
	{
		p[5,] <- y.mean;
		p[6,] <- y.sd;
		rownames(p) <- c("p.symmetric","p.equaltail","p.uppertail","p.lowertail","y.mean","y.sd");
	}
	else if (is.na(a.null)) { p$alpha <- NULL; } # If there is no null hypothesis for alpha, drop it.
	else if (is.na(r.null)) { p$rho <- NULL; }   # If there is no null hypothesis for rho, drop it.

	# If plot option is enabled, plot the distribution generated from the bootstrap under the null hypothesis.
	if (plot)
	{
		# Create a new window for plotting. 
		dev.new(width=12, height=6)
		par(mfrow=c(1,2),oma=c(0,0,0,0));
		
		# Plot the density of alpha generated from the null value and the observed parameter.
		plot(density.default(y.a$alpha),xlim=c(min(c(y.a$alpha,a.hat))-.2,max(c(y.a$alpha,a.hat))+.2),main="Alpha");
		abline(v=a.hat, lty=2, col="red");
		
		# Plot the density of rho generated from the null value and the observed parameter.
		plot(density.default(y.r$rho),xlim=c(min(c(y.r$rho,r.hat))-.2,max(c(y.r$rho,r.hat))+.2),main="Rho");
		abline(v=r.hat, lty=2, col="red");
	}
	else { return(p); } # Return the p-values.
}

# Calculate the p-values for reversals from bootstrapped {alpha,rho} estimates.
boot.R <- function(s=1:60,R=19,plot=FALSE,silent=TRUE,...)
{
	# Declare function that pulls a given row (point) from reversal observations, when the index is an even multiple of the row.
	getRow <- function(y,row,mean=TRUE)
	{
		row <- row%%5;								# Take the desired row and modulus by five to get appropriate row.
		y <- y[as.integer(row.names(y))%%5==row,];	# Grab the appropriate row.
		if (mean) { y <- colMeans(y); }				# If mean is enabled, calcuate the mean.
		return(y);									# Return the mean or full subset of the desired row.
	}
	
 	# Declare function that calculates mean number of reversals by row (where rows represent points).
	# Orders the means from "closest" to "furthest" from boundary, so it reverses the order in the raw data. 
	pointMeans <- function(y) { return(as.data.frame(rbind(getRow(y,1),getRow(y,2),getRow(y,3),getRow(y,4),getRow(y,5))[5:1,])); }
	
	# Calculates the probability of an observed reversal frequency by comparing it to simulated frequencies.
	boot.test <- function(x,y,row,R)
	{
		if (row==0)															# If the row is zero, sum over all rows.
		{
			x <- data.frame(t(colMeans(x)));								# Find the observed mean frequency.
			for (i in 1:(nrow(y)/5))										# Iterate through the simulations and total every five rows.
			{
				y[i,] <- colMeans(y[i:(i+4),]);								# Find the mean of the "next five rows."
				y     <- y[-c((i+1):(i+4)),];								# Delete the "next four rows" so that all that remains in the mean.
			}
		}
		else																# If a non-zero row is requested, grab it.
		{
			x <- getRow(x,row,FALSE);										# Extract the observed frequencies for the requested row.
			y <- getRow(y,row,FALSE);										# Extract a sample of simulated frequencies for the requested row.
		}
		
		p.uppertail <- colSums(t(apply(y,1,function(z) colSums(z>=x))))/R;	# Calculate how often the observed frequency is greater than the simulated frequency.
		p.lowertail <- colSums(t(apply(y,1,function(z) colSums(z<=x))))/R;	# Calculate how often the observed frequency is less than the simulated frequency.
		p.value     <- pmin(p.uppertail,p.lowertail);						# Find the lowest p-value; this tells you if you are out of the confidence interval on either side.
																			# (Multiply this number by 2 to conduct an "equal-tailed" test.)
		
	#	row <- rep(row,2);													# Stretch the row variable so that it can be placed in the data frame output.
	#	return(as.data.frame(cbind(row,rbind(p.uppertail,p.lowertail))));	# Output a data frame of p-values for the upper and lower tail.
		return(as.data.frame(t(p.value)));									# Output a data frame of p-values for the upper and lower tail.
	}
	
	# Gather MLE estimates for each subject.
	y <- mle.A(s=s,...);

	# Fetch information on the frequency of observed reversals.
	x <- data.frame();																					# Create a new data frame to store observed frequencies of reversal.
	for (i in 1:length(s)) { x <- rbind(x,reversal.B(s=s[i],a=y$alpha[i],r=y$rho[i],prob=FALSE)); }		# Iterate through the selected subjects (all subjects by default).
	x <- pointMeans(x);																					# Get means for each of the points.
#	row.names(x) <- 1:5																					# Make the row names numeric to ensure getRow will work.

	# Conduct simulations to generate information on the theoretical distribution of reversals.
	y.R <- data.frame();																				# Create a new data frame to store simulated frequencies of reversal.
	for (i in 1:R)																						# Iterate through replications, randomly generating reversal numbers.
	{
		if (!silent) { print(paste("Conducting bootstrap simulation:",i)); } 							# If the silent switch is off, report the current simulation to the user.
		y.j <- data.frame();																			# Create space "per replication" to store randomly generated reversals.
		for (j in 1:length(s)) { y.j <- rbind(y.j,reversal.B(a=y$alpha[j],r=y$rho[j],prob=FALSE)); }	# Calculate reversals for a random sample w/ estimated {alpha,rho}.
		y.R <- rbind(y.R,pointMeans(y.j))																# Add the means for the points to the simulated data.
	}
	
	# Calculate the p-values for the bootstrap.
	p <- data.frame();														# Create a new data frame to store the p-values.
	for (i in 0:5) { p <- rbind(p,boot.test(x,y.R,i,R)); }					# Calculate the p-value for each point using the boot.test function.
	rownames(p) <- 0:5														# Reassign row names to begin at zero to indicate the sum of all rows.

	# Give the parameters longer, more meaningful names before they are output.
	p.values  <- p;
	reversals <- x;
	simulated <- y.R;
	mle       <- y;
	
	# Return the calculated values.
	return(list(p.values,reversals,simulated,mle))
}

# Bootstrap tests: KS and Correlation.
boot.all <- function(t=c("CE","CE","PE","PE","BC","BC"),alt=c("PE","BC","BC","CE","CE","PE"),R=9999,save=TRUE,...)
{
	# Make certain that there exists a pair of institutions for each test.
	if (length(t)!=length(alt)) { stop("The argument \"t\" and \"alt\" must be of the same length."); }

	t1 <- t1=proc.time();

	# Iterate through treatment pairs and conduct bootstrapped tests.
	for (i in 1:length(t))
	{
		ks.alpha <- boot.test(t=t[i],alt=alt[i],R=R,test="ks.test",param="alpha",save=save,...);	# Kolmogorov-Smirnov test on alpha.
		ks.rho   <- boot.test(t=t[i],alt=alt[i],R=R,test="ks.test",param="rho",save=save,...);		# Kolmogorov-Smirnov test on rho.
		cs.alpha <- boot.test(t=t[i],alt=alt[i],R=R,test="cor.test",param="alpha",save=save,...);	# Spearman correlation test on alpha.
		cs.rho   <- boot.test(t=t[i],alt=alt[i],R=R,test="cor.test",param="rho",save=save,...);		# Spearman correlation test on rho.
	}
	
	print(proc.time()-t1)[3];
}

# Bootstrap the parameters.
boot.cor <- function(t=c("CE","CE","PE"),alt=c("PE","BC","BC"),R=9999,save=TRUE,...)
{
	# Make certain that there exists a pair of institutions for each test.
	if (length(t)!=length(alt)) { stop("The argument \"t\" and \"alt\" must be of the same length."); }

	# Iterate through treatment pairs and conduct bootstrapped tests.
	for (i in 1:length(t))
	{
		bs.alpha <- boot.test(t=t[i],alt=alt[i],R=R,test="boot",param="alpha",save=save,...);	# Spearman correlation test on alpha.
		bs.rho   <- boot.test(t=t[i],alt=alt[i],R=R,test="boot",param="rho",save=save,...);		# Spearman correlation test on rho.
	}
}

# Load and display bootstraps.
boot.disp <- function(t=c("CE","CE","PE","PE","BC","BC"),alt=c("PE","BC","BC","CE","CE","PE"),raw=FALSE)
{
	# Make certain that there exists a pair of institutions for each test.
	if (length(t)!=length(alt)) { stop("The argument \"t\" and \"alt\" must be of the same length."); }

	# Declare a function to load and display saved output.
	display.output <- function(t,alt,test,param,raw)
	{
		load(file=paste("boot_",t,"_",alt,"_",substr(test,1,1),"s_",param,".Rdata",sep=""))
		if (!raw) { writeLines(x$output); }
		else      { print(x$test); }
	}
	
	# Iterate through treatment pairs and conduct bootstrapped tests.
	for (i in 1:length(t))
	{
		display.output(t=t[i],alt=alt[i],test="ks.test",param="alpha",raw);		# Kolmogorov-Smirnov test on alpha.
		display.output(t=t[i],alt=alt[i],test="ks.test",param="rho",raw);		# Kolmogorov-Smirnov test on rho.
		display.output(t=t[i],alt=alt[i],test="cor.test",param="alpha",raw);	# Spearman correlation test on alpha.
		display.output(t=t[i],alt=alt[i],test="cor.test",param="rho",raw);		# Spearman correlation test on rho.
	}
}

# Display bootstrap output.
disp.cor <- function(t=c("CE","CE","PE"),alt=c("PE","BC","BC"),plot=FALSE)
{
	# Make certain that there exists a pair of institutions for each test.
	if (length(t)!=length(alt)) { stop("The argument \"t\" and \"alt\" must be of the same length."); }

	# Declare a function to load and display saved output.
	display.output <- function(t,alt,test,param)
	{
		load(file=paste("boot_",t,"_",alt,"_",substr(test,1,1),"s_",param,".Rdata",sep=""))
		writeLines(x$output);
	}
	
	# Iterate through treatment pairs and conduct bootstrapped tests.
	for (i in 1:length(t))
	{
		display.output(t=t[i],alt=alt[i],test="boot",param="alpha");	# Spearman correlation test on alpha.
		display.output(t=t[i],alt=alt[i],test="boot",param="rho");		# Spearman correlation test on rho.
	}
}

# Load bootstrap output.
boot.load <- function(t,alt,test,param)
{
	# Make certain that there exists a pair of institutions for each test and check other arguments.
	if (length(t)!=1|length(alt)!=1) { stop("The argument \"t\" and \"alt\" must both be of length one."); }
	if (!(param=="alpha"|param=="rho")) { stop("The argument \"param\" must take the value \"alpha\" or \"rho\"."); }
	if (!(test=="ks.test"|test=="cor.test"|test=="boot")) { stop("The argument \"test\" must take the value \"ks.test\", \"cor.test\", or \"boot\"."); }

	# Load the data file for the particular test and return the "boot" object.
	load(file=paste("boot_",t,"_",alt,"_",substr(test,1,1),"s_",param,".Rdata",sep=""))
	return(x)
}

# Declare a function that conducts bootstrapped tests on estimated parameters.
# The function generates values in alt following the null hypothesis for the given test.
# The library "boot" is utilized, and takes parallelization commands that may be specified as arguments.
# (The default is one core; this will take a very long time for R>99. Use better hardware.)
boot.test <- function(t,alt,R,test,param,silent=FALSE,save=FALSE,parallel="multicore",ncpus=1,...)
{
	# Load bootstrap library and double-check arguments.
	if (!require(boot,quietly=TRUE)) { stop("The library \"boot\" is required to use this function."); }
	if (!(param=="alpha"|param=="rho")) { stop("The argument \"param\" must take the value \"alpha\" or \"rho\"."); }
	if (!(test=="ks.test"|test=="cor.test"|test=="boot")) { stop("The argument \"test\" must take the value \"ks.test\", \"cor.test\", or \"boot\"."); }
	
	# If save is enabled, make sure that the file does not already exist.
	# (There is no overwrite option. To run a new bootstrap, set the save argument to FALSE or move/delete the existing files.)
	if (save)
	{
		filename <- paste("boot_",t,"_",alt,"_",substr(test,1,1),"s_",param,sep="");						# Declare filename (also used later.)
		if (file.exists(paste(filename,".Rdata",sep="")))													# Check if file already exists.
		{
			warning(paste("The file ",filename,".Rdata already exists; call has been skipped.",sep=""));	# Warn subject that call will be skipped. 
			return(FALSE);																					# Return false so that algorithm can continue.
		}
	}

	# Gather MLE estimates for each subject from treatments "t" and "alt".
	y.t   <- mle.A(t=t,...);
	y.alt <- mle.A(t=alt,...);

	# Discard observations for subjects whose parameters are inestimable (which may happen if omit is triggered).
	s     <- (1:dim(y.t)[1])[!is.na(y.t$alpha)&!is.na(y.t$rho)&!is.na(y.alt$alpha)&!is.na(y.alt$rho)];
	y.t   <- y.t[s,];
	y.alt <- y.alt[s,];
	
	# Create a new data frame of estimates from treatments "t" and "alt" and a parameter vector.
	y <- data.frame("t.alpha"=y.t$alpha,"t.rho"=y.t$rho,"alt.alpha"=y.alt$alpha,"alt.rho"=y.alt$rho);
	p <- list("n"=length(s),"t"=t,"alt"=alt);
	
	# Based on the test to be performed and parameter to be evaluated, conduct the test and declare the functions boot.gen and boot.stat.
	if (test=="ks.test")
	{
		# Conduct the Kolmorogov-Smirnov test on the data.
		if (param=="alpha")    { x.test <- ks.test(y$t.alpha,y$alt.alpha); }
		else if (param=="rho") { x.test <- ks.test(y$t.rho,y$alt.rho); }
	
		# Generate data under the Kolmorogov-Smirnov null hypothesis that the distributions are the same using fetch.B.
		boot.gen <- function(y,p)
		{
			z <- y;																								# Create a new data set to store simulated data.
			for (i in 1:p$n)																					# Iterate through every subject.
			{	
				z[i,1:2] <- mle.B(fetch.B(s=NA,t=p$t,a=y$t.alpha[i],r=y$t.rho[i]),ll=FALSE);					# Resample and generate new estimates for treatment "t".
				z[i,3:4] <- mle.B(fetch.B(s=NA,t=p$alt,a=y$t.alpha[i],r=y$t.rho[i]),ll=FALSE);					# In KS, the distributions are the same under H0.
			}
			return(z);																							# Return the simulated data.
		}
	
		# Declare function to conduct the Kolmorogov-Smirnov test on the data and return the test statistic.
		if (param=="alpha")    { boot.stat <- function(y) { return(ks.test(y$t.alpha,y$alt.alpha)$statistic); } }
		else if (param=="rho") { boot.stat <- function(y) { return(ks.test(y$t.rho,y$alt.rho)$statistic); } }
	}
	else if (test=="cor.test")
	{
		# Conduct the Spearman Rank Correlation Test on the data.
		if (param=="alpha")    { x.test <- cor.test(y$t.alpha,y$alt.alpha,method="spearman"); }
		else if (param=="rho") { x.test <- cor.test(y$t.rho,y$alt.rho,method="spearman"); }
	
		# Generate data under the Spearman Rank Correlation Test null hypothesis that the distributions are not monotonically related using fetch.B.
		boot.gen <- function(y,p)
		{
			z <- y;																								# Create a new data set to store simulated data.
			for (i in 1:p$n)																					# Iterate through every subject.
			{	
				rand     <- order(runif(p$n));
				z[i,1:2] <- mle.B(fetch.B(s=NA,t=p$t,a=y$t.alpha[i],r=y$t.rho[i]),ll=FALSE);					# Resample and generate new estimates for treatment "t".
				z[i,3:4] <- mle.B(fetch.B(s=NA,t=p$alt,a=y$alt.alpha[rand[i]],r=y$alt.rho[rand[i]]),ll=FALSE);	# In CS, the parameters are uncorrelated in rank under H0.
			}
			return(z);																							# Return the simulated data.
		}

		# Declare function to conduct the Spearman Correlation test on the data and return the test statistic.
		if (param=="alpha")    { boot.stat <- function(y) { return(cor.test(y$t.alpha,y$alt.alpha,method="spearman")$statistic); } }
		else if (param=="rho") { boot.stat <- function(y) { return(cor.test(y$t.rho,y$alt.rho,method="spearman")$statistic); } }
	}
	else if (test=="boot")
	{
		# Conduct the Spearman Rank Correlation Test on the data.
		if (param=="alpha")    { x.test <- cor.test(y$t.alpha,y$alt.alpha,method="spearman"); }
		else if (param=="rho") { x.test <- cor.test(y$t.rho,y$alt.rho,method="spearman"); }

		# Generate data under the Spearman Rank Correlation Test null hypothesis that the distributions are not monotonically related using fetch.B.
		boot.gen <- function(y,p)
		{
			z <- y;																								# Create a new data set to store simulated data.
			for (i in 1:p$n)																					# Iterate through every subject.
			{	
				rand     <- order(runif(p$n));
				z[i,1:2] <- mle.B(fetch.B(s=NA,t=p$t,a=y$t.alpha[i],r=y$t.rho[i]),ll=FALSE);					# Resample and generate new estimates for treatment "t".
				z[i,3:4] <- mle.B(fetch.B(s=NA,t=p$alt,a=y$alt.alpha[i],r=y$alt.rho[i]),ll=FALSE);				# Resample and generate new estimates for treatment "alt".
			}
			return(z);																							# Return the simulated data.
		}

		# Declare function to conduct the Spearman Correlation test on the data and return the test statistic.
		if (param=="alpha")    { boot.stat <- function(y) { return(cor(y$t.alpha,y$alt.alpha,method="spearman")); } }
		else if (param=="rho") { boot.stat <- function(y) { return(cor(y$t.rho,y$alt.rho,method="spearman")); } }
	}

	# Return the results of a parametric bootstrap.
	x <- boot(data=y,statistic=boot.stat,R=R,sim="parametric",ran.gen=boot.gen,mle=p,parallel=parallel,ncpus=ncpus);
	
	# Calculate the p-value implied by the bootstrap.
	if (test=="ks.test") { x$p.value <- (sum(x$t>=x$t0)+1)/(R+1); }       # Larger values of D are less likely under the null hypothesis.
	else if (test=="cor.test") { x$p.value <- (sum(x$t<=x$t0)+1)/(R+1); } # Smaller values of S are less likely under the null hypothesis.
	
	# Copy relevant information from the test into bootstrap output.
	x$test        <- x.test;
	x$alternative <- x$test$alternative
	x$method      <- x$test$method
	x$output      <- paste("\n	",x$method,"\n\nparameter: ",param,", treatments: ",t," and ",alt,"\nparametric bootstrap, R = ",R,", N = ",length(s),"\n",sep="");
	
	# Create an output string to display results (and copy additional info as required).
	if (test=="ks.test") { x$output <- paste(x$output,"D = ",round(x$t0,4),", p-value = ",round(x$p.value,4),"\nalternative hypothesis: ",x$alternative,"\n",sep=""); }
	else if (test=="cor.test")
	{
		x$null.value <- x$test$null.value;
		x$estimate   <- x$test$estimate;
		x$output     <- paste(x$output,"S = ",round(x$t0,4),", p-value = ",round(x$p.value,4),"\nalternative hypothesis: true rho is not equal to 0 \nsample estimates: \nrho \n   ",round(x$estimate,4),"\n",sep="");
	}
	else if (test=="boot") { x$output <- capture.output(x); }
	
	# Display results, save the data, and return bootstrap output.
	if (!silent) { writeLines(x$output); }
	if (save)
	{
		filename <- paste("boot_",t,"_",alt,"_",substr(test,1,1),"s_",param,sep="");				# Declare filename in standardized form (e.g. "boot_CE_PE_ks_alpha").
		capture.output(writeLines(x$output),file=paste("boot_output",".txt",sep=""),append=TRUE)	# Write relevant output to the file "boot_output.txt".
		dput(x,file=paste(filename,".txt",sep=""));													# Write ASCII text representation of bootstrap object to file w/ ".txt" extension.
		save(x,file=paste(filename,".Rdata",sep=""));												# Write binary R dump of bootstrap object to file w/ ".Rdata" extension.
	}
	return(x);
}

######################################################################################################################
# Declare functions for calculating reversals and reversal probabilities per the Blavatskyy model.					 #
######################################################################################################################

# Finds the unconditional probability of a choosing the p-bet in PE for index "i".
# This is E[p(P,D)] = ∫∫ p(P(q_p),D(q_d))PE_D'(q_p)PE_P'(q_d)d(q_p)d(q_d).
reversal.PE <- function(a,r,i=1:5,R=0,endogenize=TRUE,cubature=FALSE,stope=FALSE,...)
{	
	# Load required packages.
	if (cubature) { require(cubature); }																																			# Cubature does multivariate integration.

	# Pull parameters necessary for calculations and create space to store results.
	x <- param.B()[,1:4];																																							# Fetch relevant parameters.
	x[c("HPE","BPE","RPE")] <- 0;																																					# Create space for generated data.
	
	# Declare functions and make calculations required for endogenization.
	b <- function(p) { return(pmax(pmin(p,rep(1,length(p))),rep(0,length(p)))) }																									# Probabilities must lie in [0,1].
	PPE <- rep(0,22);																																								# Create a vector to store bitter/sweetness.
	if (endogenize)
	{
		p <-  param.B();																																							# Get full set of parameters. 
		PPE[1:5]   <- (as.numeric(x$Hupfront[1:5]>=x$Hupfront[11:15])*p$SBPE[1:5]-as.numeric(x$Hupfront[1:5]<x$Hupfront[11:15])*p$BBPE[1:5])/100;									# (Exogenously) bitter/sweeten p-bet.
		PPE[11:15] <- (as.numeric(x$Hupfront[1:5]<x$Hupfront[11:15])*p$SBPE[11:15]-as.numeric(x$Hupfront[1:5]>=x$Hupfront[11:15])*p$BBPE[11:15])/100;								# (Exogenously) bitter/sweeten $-bet.
		if (R<=0) { F.e <- function(A,B,pA,pB,a,r,eA,eB) { return(F(A,B,b(pA+eA),b(pB+eB),a,r)); } }																				# Find "endogenized" CDF.
	}
	else if (R<=0) { F.e <- function(A,B,pA,pB,a,r,eA,eB) { return(F(A,B,pA,pB,a,r)); }	}																							# Find "non-endogenized" CDF.
	
	# By default, calculate the numerical probability of reversal.
	if (R<=0)
	{		
		# If cubature option is not selected, embed one integral into the other and calculate the probability using built-in numerical integration.
		# This is generally slightly faster than cubature. "Stop on error" has been turned off by default to allow complex calculations.
		# However, the integral may be *very* slow when it's nasty (e.g. alpha < 1) – it's best to use cubature, or even bootstrapping, in those cases.
		if (!cubature)
		{
			g <- function(pA,pB,A,B,uA,uB,a,r,eA,eB) { return(F.e(A,B,pA,pB,a,r,eA,eB)*f(A,uA,pA,1,a,r)*f(B,uB,pB,1,a,r)); }														# Function to be integrated, pA as first argument.
			inner <- function(pB,A,B,uA,uB,a,r,eA,eB) { return(integrate(Vectorize(g),1e-16,1-1e-16,pB=pB,A=A,B=B,uA=uA,uB=uB,a=a,r=r,eA=eA,eB=eB,stop.on.error=stope)$value); } 	# Inner integral over pA, pB as first argument (given).
			for (j in i) { x$BPE[j] <- integrate(Vectorize(inner),1e-16,1-1e-16,A=x$HoutcomeA[j],B=x$HoutcomeA[j+10],uA=x$Hupfront[j],uB=x$Hupfront[j+10],a=a,r=r,eA=PPE[j],eB=PPE[j+10],stop.on.error=stope)$value; }
		}
		# If cubature option is selected, use the multivariate integration function from that package.
		# This is not necessarily faster, but works when (alpha < 1) – the preceding approach tends to diverge when it hits those kinds of nasty integrals.
		else
		{
			g <- function(p,A,B,uA,uB,a,r,eA,eB) { return(F.e(A,B,p[1],p[2],a,r,eA,eB)*f(A,uA,p[1],1,a,r)*f(B,uB,p[2],1,a,r)); }													# Function to be integrated, p=c(pA,pB) as vector.
			for (j in i) { x$BPE[j] <- adaptIntegrate(g,rep(1e-16,2),rep(1-1e-16,2),A=x$HoutcomeA[j],B=x$HoutcomeA[j+10],uA=x$Hupfront[j],uB=x$Hupfront[j+10],a=a,r=r,eA=PPE[j],eB=PPE[j+10])$integral; }
		}
		#	x$BPE[11:15] <- 1-x$BPE[1:5];																																			# Prob. of $-bet is 1-p-bet.
		#	x$BPE[c(6:10,16:22)] <- NA;																																				# Let irrelevant indices be NA.
	}
	
	# If R>0, instead bootstrap the probability (for comparison).
	else 																
	{
		sample.PE <- function(a,r,i,PPE)																																			# Declare function that generates sample data.
		{
			
			x <- param.B()[,1:4];																																					# Fetch relevant parameters.
			x[c("HPE","BPE","RPE")] <- NA																																			# Create space for generated data.
			x$HPE[c(i,i+10)] <- sampleF.p(x$HoutcomeA[c(i,i+10)],x$Hupfront[c(i,i+10)],rep(1,2*length(i)),a,r)*100;																	# Generate probability equivalents.
			x$BPE[i]         <- as.numeric(F(x$HoutcomeA[i],x$HoutcomeA[i+10],b(x$HPE[i]/100+PPE[i]),b(x$HPE[i+10]/100+PPE[i+10]),a,r)>runif(length(i)));							# Generate binary choices for p-bets.
			# x$BPE[i+10]    <- as.numeric(!x$BPE[i]);																																# Generate binary choices for $-bets.
			return(x)
		}
		for (j in 1:R) { x <- ((j-1)/j)*x+(1/j)*sample.PE(a=a,r=r,i=i,PPE=PPE); }																									# Average R sample replications.
	}
	
	# Return calculated probabilities.
	return(x$BPE[i])
}

# Finds the unconditional probability of a choosing the p-bet in PL for index "i".
# This is E[p(P,D)] = ∫∫ p(P(q_p),D(q_d))PE_D'(q_p)PE_P'(q_d)d(q_p)d(q_d).
# Note that PE_D and PE_P are truncated distributions, and f(x|X≤y)=g(x)/F(x), as defined in the f.tr function.
reversal.PL <- function(a=2,r=.25,i=6:10,R=0,endogenize=TRUE,cubature=FALSE,stope=FALSE,...)
{	
	# Load required packages.
	if (cubature) { require(cubature); }																																			# Cubature does multivariate integration.

	# Pull parameters necessary for calculations and create space to store results.
	x <- param.B()[,1:4];																																							# Fetch relevant parameters.
	x[c("HPL","BPL","RPL")] <- 0;																																					# Create space for generated data.
	
	# Declare functions and make calculations required for endogenization.
	b <- function(p) { return(pmax(pmin(p,rep(1,length(p))),rep(0,length(p)))) }																									# Probabilities must lie in [0,1].
	PPL <- rep(0,22);																																								# Create a vector to store bitter/sweetness.
	if (endogenize)
	{
		p <-  param.B();																																							# Get full set of parameters. 
		PPL[6:10]   <- (as.numeric(x$HrefprobA[6:10]>=x$HrefprobA[16:20])*p$SBPL[6:10]-as.numeric(x$HrefprobA[6:10]<x$HrefprobA[16:20])*p$BBPL[6:10])/100;							# (Exogenously) bitter/sweeten p-bet.
		PPL[16:20] <- (as.numeric(x$HrefprobA[6:10]<x$HrefprobA[16:20])*p$SBPL[16:20]-as.numeric(x$HrefprobA[6:10]>=x$HrefprobA[16:20])*p$BBPL[16:20])/100;							# (Exogenously) bitter/sweeten $-bet.
		if (R<=0) { F.e <- function(A,B,pA,pB,a,r,eA,eB) { return(F(A,B,b(pA+eA),b(pB+eB),a,r)); } }																				# Find "endogenized" CDF.
	}
	else if (R<=0) { F.e <- function(A,B,pA,pB,a,r,eA,eB) { return(F(A,B,pA,pB,a,r)); }	}																							# Find "non-endogenized" CDF.
	
	# By default, calculate the numerical probability of reversal.
	if (R<=0)
	{		
		# If cubature option is not selected, embed one integral into the other and calculate the probability using built-in numerical integration.
		# This is generally slightly faster than cubature. "Stop on error" has been turned off by default to allow complex calculations.
		# However, the integral may be *very* slow when it's nasty (e.g. alpha < 1) – it's best to use cubature, or even bootstrapping, in those cases.
		# We turn on filter here; otherwise the function returns NaN as some points where the pdf is zero. 
		if (!cubature)
		{	

			g <- function(pA,pB,A,B,qA,qB,a,r,eA,eB) { return(F.e(A,B,pA,pB,a,r,eA,eB)*f.tr(pA,qA,A,80,a,r)*f.tr(pB,qB,B,80,a,r)); }												# Function to be integrated, pA as first argument.			
			inner <- function(pB,A,B,qA,qB,a,r,eA,eB) { return(integrate(Vectorize(g),qA,1-1e-16,pB=pB,A=A,B=B,qA=qA,qB=qB,a=a,r=r,eA=eA,eB=eB,stop.on.error=stope)$value); } 		# Inner integral over pA, pB as first argument (given).
			for (j in i) { x$BPL[j] <- integrate(Vectorize(inner),x$HrefprobA[j+10]/100,1-1e-16,A=x$HoutcomeA[j],B=x$HoutcomeA[j+10],qA=x$HrefprobA[j]/100,qB=x$HrefprobA[j+10]/100,a=a,r=r,eA=PPL[j],eB=PPL[j+10],stop.on.error=stope)$value; }
				
		}
		# If cubature option is selected, use the multivariate integration function from that package.
		# This is not necessarily faster, but works when (alpha < 1) – the preceding approach tends to diverge when it hits those kinds of nasty integrals.
		else
		{
			g <- function(p,A,B,qA,qB,a,r,eA,eB) { return(F.e(A,B,p[1],p[2],a,r,eA,eB)*f.tr(p[1],qA,A,80,a,r)*f.tr(p[2],qB,B,80,a,r)); }											# Function to be integrated, p=c(pA,pB) as vector.
			for (j in i) { x$BPL[j] <- adaptIntegrate(g,c(x$HrefprobA[j],x$HrefprobA[j+10])/100,rep(1-1e-16,2),A=x$HoutcomeA[j],B=x$HoutcomeA[j+10],qA=x$HrefprobA[j]/100,qB=x$HrefprobA[j+10]/100,a=a,r=r,eA=PPL[j],eB=PPL[j+10])$integral; }
		}
	}
	
	# If R>0, instead bootstrap the probability (for comparison).
	else 																
	{
		sample.PL <- function(a,r,i,PPL)																																			# Declare function that generates sample data.
		{
			
			x <- param.B()[,1:4];																																					# Fetch relevant parameters.
			x[c("HPL","BPL","RPL")] <- NA																																			# Create space for generated data.
			x$HPL[c(i,i+10)] <- sampleF.p(x$HoutcomeA[c(i,i+10)],rep(80,2*length(i)),x$HrefprobA[c(i,i+10)]/100,a,r)*100;															# Generate probability equivalents.
			x$BPL[i]         <- as.numeric(F(x$HoutcomeA[i],x$HoutcomeA[i+10],b(x$HPL[i]/100+PPL[i]),b(x$HPL[i+10]/100+PPL[i+10]),a,r)>runif(length(i)));							# Generate binary choices for p-bets.
			# x$BPL[i+10]      <- as.numeric(!x$BPL[i]);																															# Generate binary choices for $-bets.
			return(x)
		}
		for (j in 1:R) { x <- ((j-1)/j)*x+(1/j)*sample.PL(a=a,r=r,i=i,PPL=PPL); }																									# Average R sample replications.
	}
	
	# Return calculated probabilities.
	return(x$BPL[i])
}

# Define a function that returns information about reversals and reversal probabilities.
# For partial, use 0 for full, 1 for BDM constant, and 2 for BC constant.
reversal.B <- function(s=NA,a=NA,r=NA,strong=FALSE,endogenize=TRUE,prob=TRUE,cons=FALSE,type=FALSE)
{
	# Load parameters and data for selected subject or a random agent w/ {alpha,rho}.
	p <- param.B();
	if(is.na(s)) { x <- sample.B(a=a,r=r,endogenize); }
	else { x <- disp.B(s); }
	
	# If doing consistency checks, copy the consistency responses over the main responses.
	if (cons)
	{
		x$HCE  <- x$RCE;
		x$HPE  <- x$RPE;
		x$HPL  <- x$RPL;
		x$HPR  <- x$RPR;
	}
	
	# Get estimated alpha and rho for subject if not specified.
	if (prob&(is.na(a)|is.na(r)))
	{
		y <- mle.B(fetch.B(s,t=c("CE","PE","BC")));
		if (is.na(a)) { a = y[1]; }
		if (is.na(r)) { r = y[2]; }
	}
	
	if (prob)
	{
		# Create space to store endogenization adjustments.
		PCE <- rep(0,22);
		PPE <- rep(0,22);
		PPL <- rep(0,22);
		PPR <- rep(0,22);
	
		# If endogenization is triggered (as default), endoegenize choices for binary choice.
		if (endogenize)
		{
			# Create vectors of endogeneity adjustments.
			PCE[1:5]   <- as.numeric(x$HCE[1:5]>=x$HCE[11:15])*p$SBCE[1:5]-as.numeric(x$HCE[1:5]<x$HCE[11:15])*p$BBCE[1:5];
			PCE[11:15] <- as.numeric(x$HCE[1:5]<x$HCE[11:15])*p$SBCE[11:15]-as.numeric(x$HCE[1:5]>=x$HCE[11:15])*p$BBCE[11:15];
			PPE[1:5]   <- as.numeric(x$Hupfront[1:5]>=x$Hupfront[11:15])*p$SBPE[1:5]-as.numeric(x$Hupfront[1:5]<x$Hupfront[11:15])*p$BBPE[1:5];
			PPE[11:15] <- as.numeric(x$Hupfront[1:5]<x$Hupfront[11:15])*p$SBPE[11:15]-as.numeric(x$Hupfront[1:5]>=x$Hupfront[11:15])*p$BBPE[11:15];
			PPL[6:10]  <- as.numeric(x$HrefprobA[6:10]>=x$HrefprobA[16:20])*p$SBPL[6:10]-as.numeric(x$HrefprobA[6:10]<x$HrefprobA[16:20])*p$BBPL[6:10];
			PPL[16:20] <- as.numeric(x$HrefprobA[6:10]<x$HrefprobA[16:20])*p$SBPL[16:20]-as.numeric(x$HrefprobA[6:10]>=x$HrefprobA[16:20])*p$BBPL[16:20];	
			PPR[6:10]  <- as.numeric(x$HPR[6:10]>=x$HPR[16:20])*p$SBPR[6:10]-as.numeric(x$HPR[6:10]<x$HPR[16:20])*p$BBPR[6:10];
			PPR[16:20] <- as.numeric(x$HPR[6:10]<x$HPR[16:20])*p$SBPR[16:20]-as.numeric(x$HPR[6:10]>=x$HPR[16:20])*p$BBPR[16:20];		
			PCE[6:10]  <- as.numeric(x$HCE[6:10]>=rep(x$HCE[12],5))*p$SBCE[1:5]-as.numeric(x$HCE[6:10]<rep(x$HCE[12],5))*p$BBCE[1:5];
			PCE[16:20] <- as.numeric(x$HCE[6:10]<rep(x$HCE[12],5))*p$SBCE[1:5]-as.numeric(x$HCE[6:10]>=rep(x$HCE[12],5))*p$BBCE[1:5];
		}
	
		# Declare function to bound binary choice probabilities that fall out of the range [0,100].
		b <- function(p) { return(pmax(pmin(p,rep(100,length(p))),rep(0,length(p)))) }
	}
	
	# Create space to store observed reversals and reversal probabilities (if applicable).
	R <- as.data.frame(matrix(NA,5,0));
	if (strong)
	{
		R[c("CE.rS","PR.rS")] <- NA;																		# Is strong reversal observed?
		if (prob)
		{
			if (type!="BDM") { R[c("CE.pS","PR.pS")] <- NA; }												# Probability of strong reversal?
			else             { R[c("CE.pS")] <- NA; }														# Probability of CE(D)>CE(P)?
		}
	}
	else
	{
		if (type!="BDM") { R[c("CE.rs","CE.rn","PE.rs","PE.rn","PL.rs","PL.rn","PR.rs","PR.rn")] <- NA; }		# Is reversal observed?
		else 			 { R[c("CE.rs","CE.rn","PR.rs","PR.rn")] <- NA; }										# Is E(D)>E(P)? (No PE or PL.)
		if (prob)
		{
			if (type!="BDM") { R[c("CE.ps","CE.pn","PE.ps","PE.pn","PL.ps","PL.pn","PR.ps","PR.pn")] <- NA; }	# Probability of observing reversal? (All.)
			else             { R[c("CE.ps","CE.pn","PR.ps","PR.pn")] <- NA; }									# Probability of E(D)>E(P)? (No PE or PL.)
		}
	}
	
	# The following probabilities are defined following Blavastkyy (2010); p3 is defined for and calculated from PR data after Butler & Loomes (2007).
	# 	Let the P-bet <=> Lottery A and the $-bet <=> Lottery B.
	#   A STANDARD preference reversal occurs when the individual chooses A over B in binary choice and values B over A in BDM elicitation.
	#   A NONSTANDARD preference reversal occurs when the individual chooses B over A in binary choice and values A over B in BDM elicitation.
	# 		p1 is the probability that an individual chooses B over A.
	#			p1 <- F(A=x$HoutcomeA[i+10],B=x$HoutcomeA[i],pA=x$HprobA[i+10]/100,pB=x$HprobA[i]/100,a=a,r=r);
	#		p2 is the probability that an individual states a higher CE for B than A.
	#			p2 <- Pr.x(A=x$HoutcomeA[i],B=x$HoutcomeA[i+10],pA=x$HprobA[i]/100,pB=x$HprobA[i+10]/100,a=a,r=r);
	#		p3 is the probability that an individual states a higher PE (in PR) for A than B.
	#			p3 <- Pr.p(A=x$HoutcomeA[i+10],B=x$HoutcomeA[i],pA=x$HprobA[i+10]/100,pB=x$HprobA[i]/100,qA=80,qB=80,a=a,r=r);
	# 	Then (1-p1)*p2 is the probability of a standard reversal in CE and p1*(1-p2) the probability of a nonstandard reversal.
	# 	Also (1-p1)*(1-p3) is the probability of a standard reversal in PR and p1*p3 the probability of a nonstandard reversal.

	# Declare functions that adjust the index for the P-bet (A) or $-bet (B).
	A <- function(i) { return(i); }
	B <- function(i) { return(i+10); }

	# Iterate through the index and count reversals and calculate probabilities.
	if (!cons) { rows <- 1:5; } else { rows <- c(2,4); }
	for  (i in rows)
	{
		# Set j for PL/PR index.
		j <- i+5;

		if (type=="BC")
		{
			# Determine whether or not the subject chose the p-bet or $-bet in reversal in CE.
			if (strong) { R$CE.rS[i] <- as.numeric(x$BCE[A(i)]); }
			else
			{
				R$CE.rs[i] <- as.numeric(x$BCE[A(i)]);
				R$CE.rn[i] <- as.numeric(x$BCE[B(i)]);
			}

			# Determine whether or not the subject chose the p-bet or $-bet in reversal in PE.
			if (!strong)
			{
				R$PE.rs[i] <- as.numeric(x$BPE[A(i)]);
				R$PE.rn[i] <- as.numeric(x$BPE[B(i)]);
			}
	
			# Determine whether or not the subject chose the p-bet or $-bet in reversal in PL.
			if (!strong)
			{
				R$PL.rs[i] <- as.numeric(x$BPL[A(j)]);
				R$PL.rn[i] <- as.numeric(x$BPL[B(j)]);
			}
		
			# Determine whether or not the subject chose the p-bet or $-bet in reversal in PR.
			if (strong) { R$PR.rS[i] <- as.numeric(x$BPR[B(j)]); }
			else
			{
				R$PR.rs[i] <- as.numeric(x$BPR[A(j)]);
				R$PR.rn[i] <- as.numeric(x$BPR[B(j)]);
			}
		}
		else if (type=="BDM")
		{
			# Determine whether or not the subject gave a higher CE elicitation for the p-bet or $-bet.
			if (strong) { R$CE.rS[i] <- as.numeric((x$Houtcome[A(i)]<x$HCE[B(i)])); }
			else
			{
				R$CE.rs[i] <- as.numeric((x$HCE[A(i)]<x$HCE[B(i)]));
				R$CE.rn[i] <- as.numeric((x$HCE[A(i)]>x$HCE[B(i)]));
			}

		#	# Determine whether or not the subject gave a higher PE elicitation for the p-bet or $-bet.
		#	R$PE.rs[i] <- as.numeric(x$HPE[A(i)]>x$HPE[B(i)]);
		#	R$PE.rn[i] <- as.numeric(x$HPE[A(i)]<x$HPE[B(i)]);

		#	# Determine whether or not the subject gave a higher PL elicitation for the p-bet or $-bet.
		#	R$PL.rs[i] <- as.numeric(x$HPL[A(j)]>x$HPL[B(j)]);
		#	R$PL.rn[i] <- as.numeric(x$HPL[A(j)]<x$HPL[B(j)]);
	
			# Determine whether or not the subject gave a higher PR elicitation for the p-bet or $-bet.'
			if (strong) { as.numeric(x$HPR[A(j)]>x$HprobA[B(j)]); }
			else
			{
				R$PR.rs[i] <- as.numeric(x$HPR[A(j)]<x$HPR[B(j)]);
				R$PR.rn[i] <- as.numeric(x$HPR[A(j)]>x$HPR[B(j)]);
			}
		}
		else
		{
			# Determine whether or not the subject engaged in reversal in CE.
			# Note that ties in CE were always broken in the program in "favor" of the p-bet in BC perturbation.
			if (strong) { R$CE.rS[i] <- as.numeric(x$BCE[A(i)]&(x$Houtcome[A(i)]<x$HCE[B(i)])); }
			else
			{
				R$CE.rs[i] <- as.numeric(x$BCE[A(i)]&(x$HCE[A(i)]<x$HCE[B(i)]));
				R$CE.rn[i] <- as.numeric(x$BCE[B(i)]&(x$HCE[A(i)]>=x$HCE[B(i)]));
			}

			# Determine whether or not the subject engaged in reversal in PE. (No ties possible.)
			if (!strong)
			{
				R$PE.rs[i] <- as.numeric(x$BPE[A(i)]&(x$Hupfront[A(i)]<x$Hupfront[B(i)]));
				R$PE.rn[i] <- as.numeric(x$BPE[B(i)]&(x$Hupfront[A(i)]>x$Hupfront[B(i)]));
			}
			# R$PE.rL[i] <- as.numeric(x$HPE[A(i)]>x$HPE[B(i)]);
	
			# Determine whether or not the subject engaged in reversal in PL. (No ties possible.)
			if (!strong)
			{
				R$PL.rs[i] <- as.numeric(x$BPL[A(j)]&(x$HrefprobA[A(j)]<x$HrefprobA[B(j)]));
				R$PL.rn[i] <- as.numeric(x$BPL[B(j)]&(x$HrefprobA[A(j)]>x$HrefprobA[B(j)]));
			}
		
			# Determine whether or not the subject engaged in reversal in PR.
			# Note that ties in PR were always broken in the program in "favor" of the p-bet in BC perturbation.
			if (strong) { R$PR.rS[i] <- as.numeric(x$BPR[B(j)]&(x$HPR[A(j)]>max(x$HprobA[B(j)],x$HPR[B(j)]))); }
			else
			{
				R$PR.rs[i] <- as.numeric(x$BPR[A(j)]&(x$HPR[A(j)]<x$HPR[B(j)]));
				R$PR.rn[i] <- as.numeric(x$BPR[B(j)]&(x$HPR[A(j)]>=x$HPR[B(j)]));
			}
		}

		# Calculate probabilities if applicable.
		if (prob)
		{
			# Calculate the probabilities of a binary choice in CE and PE as per Blavatskyy.
			# Note that p2 = ∫P(x,A)dP(x,B) should be integrated on [0,r].
			if (type!="BDM")
			{
				p.BC <- F(A=x$HoutcomeA[B(i)],B=x$HoutcomeA[A(i)],pA=b(x$HprobA[B(i)]+PCE[B(i)])/100,pB=b(x$HprobA[A(i)]+PCE[A(i)])/100,a=a,r=r);		# p1 for CE
				p.BR <- F(A=x$HoutcomeA[B(j)],B=x$HoutcomeA[A(j)],pA=b(x$HprobA[B(j)]+PPR[B(j)])/100,pB=b(x$HprobA[A(j)]+PPR[A(j)])/100,a,r); 			# p1 for PR
				# if (type=="BC")
				# {
				# 	p.BU <- F(A=x$HoutcomeA[B(i)],B=x$HoutcomeA[A(i)],pA=b(x$HPE[B(i)]+PPE[B(i)])/100,pB=b(x$HPE[A(i)]+PPE[A(i)])/100,a=a,r=r);			# p1 for PE (conditional)
				# 	p.BL <- F(A=x$HoutcomeA[B(j)],B=x$HoutcomeA[A(j)],pA=b(x$HPL[B(j)]+PPL[B(j)])/100,pB=b(x$HPL[A(j)]+PPL[A(j)])/100,a,r);				# p1 for PL (conditional)
				# }
			}

			# Calculate the probabilities of a BDM relation in PR and PL as per Blavatskyy.
			if (type!="BC")
			{
				if (strong)
				{
					p.By <- F(A=x$HoutcomeA[B(i)],B=x$HoutcomeA[A(i)],pA=x$HprobA[B(i)]/100,pB=1,a=a,r=r);												# p(B,y) for CE-strong
					p.Ay <- F(A=x$HoutcomeA[A(j)],B=80,pA=x$HprobA[A(j)]/100,pB=x$HprobA[B(j)]/100,a=a,r=r);											# p(A,y) for PR-strong
				}				
				else
				{
					p.CE <- Pr.x(A=x$HoutcomeA[A(i)],B=x$HoutcomeA[B(i)],pA=x$HprobA[A(i)]/100,pB=x$HprobA[B(i)]/100,a=a,r=r);							# p2 for CE
					p.PR <- Pr.p(A=x$HoutcomeA[B(j)],B=x$HoutcomeA[A(j)],pA=x$HprobA[B(j)]/100,pB=x$HprobA[A(j)]/100,qA=80,qB=80,a=a,r=r);				# p3 for PR
				}
			}
			
			# Calculate the expected probability that an individual will reverse in PE and PL, the "endogenous" mechanisms.
			# Use the cubature package if available for speed.
			if (!strong&type!="BC"&type!="BDM")
			{			
				if (is.element('cubature',installed.packages()[,1]))
				{
					p.PE <- reversal.PE(a,r,i,cubature=TRUE);																						# Pr(A,B) w/ {A,B}~rand
					p.PL <- reversal.PL(a,r,j,cubature=TRUE);																						# Pr(A,B) w/ {A,B}~rand (truncated)
				}
				else
				{
					p.PE <- reversal.PE(a,r,i);																										# Same, w/o cubature
					p.PL <- reversal.PL(a,r,j);																										# Same, w/o cubature (truncated)
				}
				# p.PE <- 0;
			}																																		
		
			if (type=="BC")
			{			
				# Calculate probabilities of choosing the p-bet over $-bet (as in a standard reversal) and vice versa for CE.
				if (strong) { R$CE.pS[i] <- 1-p.BC; }
				else
				{
					R$CE.ps[i] <- 1-p.BC;
					R$CE.pn[i] <- p.BC;
				}

				# Calculate probabilities of choosing the p-bet over $-bet (as in a standard reversal) and vice versa for PR.
				if (strong) { R$PR.pS[i] <- p.BR; }
				else
				{
					R$PR.ps[i] <- 1-p.BR;
					R$PR.pn[i] <- p.BR;
				}
			}
			else if (type=="BDM")
			{
				# Calculate the probability of CE_B>CE_A (CE_B>y) as in a standard reversal (and vice versa) for CE.
				if (strong) { R$CE.pS[i] <- p.By; }
				else
				{
					R$CE.ps[i] <- p.CE;
					R$CE.pn[i] <- 1-p.CE;
				}
			
				# Calculate the probability of CE_B>CE_A (CE_B>y) as in a standard reversal (and vice versa) for PR.
				if (strong) { R$PR.pS[i] <- p.Ay; }
				else
				{
					R$PR.ps[i] <- 1-p.PR;
					R$PR.pn[i] <- p.PR;
				}
	
				#	# Calculate the probability that PE_B>PE_A as in a standard reversal (and vice versa) for PE.
				#	R$PE.pS[i] <- p.PE;
				#	R$PE.ps[i] <- p.PE;
				#	R$PE.pn[i] <- 1-p.PE;
			}
			else
			{
				# Calculate the probabilities of strong, standard, and non-standard reversals in CE.		
				if (strong) { R$CE.pS[i] <- (1-p.BC)*p.By; }
				else
				{
					R$CE.ps[i] <- (1-p.BC)*p.CE;
					R$CE.pn[i] <- p.BC*(1-p.CE);
				}
		
				# Calculate the probabilities of standard and non-standard reversals in PR.
				if (strong) { R$PR.pS[i] <- p.BR*p.Ay; }
				else
				{
					R$PR.ps[i] <- (1-p.BR)*(1-p.PR);
					R$PR.pn[i] <- p.BR*p.PR;
				}
				
				if (!strong)
				{
					# Calculate the probabilities of standard and non-standard reversals in PE.
					if (x$Hupfront[B(i)]>x$Hupfront[A(i)])
					{
						R$PE.ps[i] <- p.PE;															# Probability of standard reversal where CE_B>CE_A.
						R$PE.pn[i] <- 0;															# No non-standard reversals can occur.
					}
					else
					{
						R$PE.ps[i] <- 0;															# No standard reversals can occur.
						R$PE.pn[i] <- (1-p.PE);														# Probability of non-standard reversal where CE_A>CE_B.
					}
				
					# Calculate the probabilities of standard and non-standard reversals in PL.
					if (x$Hrefprob[B(j)]>x$Hrefprob[A(j)])
					{
						R$PL.ps[i] <- p.PL;														# Probability of standard reversal where PR_B>PR_A.
						R$PL.pn[i] <- 0;														# No non-standard reversals can occur.
					}
					else
					{
						R$PL.ps[i] <- 0;														# No standard reversals can occur.
						R$PL.pn[i] <- p.PL;														# Probability of non-standard reversal where PR_A>PR_B.
					}
				}
			}
		}
	}
	
	# If doing consistency checks, eliminate NA rows.
	# if (cons) {  R <- na.omit(R); }
	
	# Return reversals (and probabilities).
	return(R);
}

reversal.A <- function(s=1:60,...)
{
	# Create a data frame to store reversal observations by subject.
	y <- data.frame();
	
	# Do MLE estimations for every subject and store them in a data frame.
	for (i in 1:length(s)) { y <- rbind(y,reversal.B(s=s[i],...)); }

	# Return the collected reversal data.
	# If reversal.B was called with the cons=TRUE (consistency checks), NA's will need to be omitted.
	return(na.omit(y));
}

# Define a function that returns information about monotonicity violations and probabilities.
# Note that it is not possible to observe monotonicity violations in the PE version of PL (as elicited in our experiment).
monotonicity.B <- function(s=NA,a=NA,r=NA,endogenize=TRUE,prob=TRUE)
{
	# Load parameters and data for selected subject or a random agent w/ {alpha,rho}.
	p <- param.B();
	if(is.na(s)) { x <- sample.B(a=a,r=r,endogenize); }
	else { x <- disp.B(s); }

	# Get estimated alpha and rho for subject if not specified.
	if(is.na(a)|is.na(r))
	{
		y <- mle.B(fetch.B(s,t=c("CE","PE","BC")));
		if (is.na(a)) { a = y[1]; }
		if (is.na(r)) { r = y[2]; }
	}

	#	   Mono.     Mono.  
	#	 Violation Violation 
	#	 (Binary)    Prob. 
	# CE    mv	      pv
	# PE	mv	  	  pv
	# PL    mv        pv
	# PR	mv	 	  pv
	
	# Create space to store calculated probabilities of monotonicity violations.
	M <- as.data.frame(matrix(NA,20,0));
	M[c("dominated","dominant","CE.mv","PE.mv","PL.mv","PR.mv","CE.pv","PE.pv","PL.pv","PR.pv")] <- NA;
	
	# Declare functions that adjust the index for the P-bet (A) or $-bet (B).
	A <- function(i) { return(i); }
	B <- function(i) { return(i+10); }
	
	# Declare function that returns all pairwise combinations of the five lotteries for each BDM treatment. Note Cn(5,2)=10.
	# The function combl returns dominated/dominant indices for PL, which are based upon reference lottery relationships.
	combn <- function(x,dominated)
	{
		if (dominated) { return(c(x[1],x[1],x[1],x[1],x[2],x[2],x[2],x[3],x[3],x[4])) }
		else           { return(c(x[2],x[3],x[4],x[5],x[3],x[4],x[5],x[4],x[5],x[5])) }
	}
	
	# Save nodes of dominant and dominated bets.
	M$dominated <- combn(1:5,1);
	M$dominant  <- combn(1:5,0);
	
	# Determine whether or not the subject engaged in a monotonicity violation.
	M$CE.mv[A(1:10)] <- as.numeric(combn(x$HCE[A(1:5)],1)>combn(x$HCE[A(1:5)],0));
	M$CE.mv[B(1:10)] <- as.numeric(combn(x$HCE[B(1:5)],1)>combn(x$HCE[B(1:5)],0));
	M$PE.mv[A(1:10)] <- as.numeric(combn(x$HPE[A(1:5)],1)>combn(x$HPE[A(1:5)],0));
	M$PE.mv[B(1:10)] <- as.numeric(combn(x$HPE[B(1:5)],1)>combn(x$HPE[B(1:5)],0));
	
	combn(p$HrefprobA[6:10],1)>combn(p$HrefprobA[6:10],0)
	M$PL.mv[A(1:10)] <- as.numeric(combn(x$HPL[A(6:10)],1)>combn(x$HPL[A(6:10)],0));
	M$PL.mv[B(1:10)] <- as.numeric(combn(x$HPL[B(6:10)],1)>combn(x$HPL[B(6:10)],0));
	
	M$PR.mv[A(1:10)] <- as.numeric(combn(x$HPR[A(6:10)],1)>combn(x$HPR[A(6:10)],0));
	M$PR.mv[B(1:10)] <- as.numeric(combn(x$HPR[B(6:10)],1)>combn(x$HPR[B(6:10)],0));
	
	if (prob)
	{
		# Calculate the probability that an individual states a higher CE/PE for the dominated initial lottery than the dominating lottery.
		for (i in 1:10)
		{
			M$CE.pv[A(i)] <- Pr.x(A=combn(x$HoutcomeA[A(1:5)],0)[i],B=combn(x$HoutcomeA[A(1:5)],1)[i],pA=combn(x$HprobA[A(1:5)]/100,0)[i],pB=combn(x$HprobA[A(1:5)]/100,1)[i],a=a,r=r);
			M$CE.pv[B(i)] <- Pr.x(A=combn(x$HoutcomeA[B(1:5)],0)[i],B=combn(x$HoutcomeA[B(1:5)],1)[i],pA=combn(x$HprobA[B(1:5)]/100,0)[i],pB=combn(x$HprobA[B(1:5)]/100,1)[i],a=a,r=r);
			M$PE.pv[A(i)] <- Pr.p(A=combn(x$Hupfront[A(1:5)],0)[i],B=combn(x$Hupfront[A(1:5)],1)[i],pA=1,pB=1,qA=combn(x$HoutcomeA[A(1:5)],0)[i],qB=combn(x$HoutcomeA[A(1:5)],1)[i],a=a,r=r);
			M$PE.pv[B(i)] <- Pr.p(A=combn(x$Hupfront[B(1:5)],0)[i],B=combn(x$Hupfront[B(1:5)],1)[i],pA=1,pB=1,qA=combn(x$HoutcomeA[B(1:5)],0)[i],qB=combn(x$HoutcomeA[B(1:5)],1)[i],a=a,r=r);
			M$PR.pv[A(i)] <- Pr.p(A=combn(x$HoutcomeA[A(6:10)],0)[i],B=combn(x$HoutcomeA[A(6:10)],1)[i],pA=combn(x$HprobA[A(6:10)]/100,0)[i],pB=combn(x$HprobA[A(6:10)]/100,1)[i],qA=80,qB=80,a=a,r=r,ub=1-1e-10);
			M$PR.pv[B(i)] <- Pr.p(A=combn(x$HoutcomeA[B(6:10)],0)[i],B=combn(x$HoutcomeA[B(6:10)],1)[i],pA=combn(x$HprobA[B(6:10)]/100,0)[i],pB=combn(x$HprobA[B(6:10)]/100,1)[i],qA=80,qB=80,a=a,r=r,ub=1-1e-10);
		}
	}

	# Return reversals and probabilities.
	return(M);
}

# Define a function that returns information about monotonicity violations and probabilities for all subjects.
monotonicity.A <- function(s=1:60,...)
{
	# Create a data frame to store reversal observations by subject.
	y <- data.frame();
	
	# Do MLE estimations for every subject and store them in a data frame.
	for (i in 1:length(s)) { y <- rbind(y,monotonicity.B(s=s[i],...)); }

	# Return the collected reversal data.
	return(y);
}

# Calculates the probability mass of reversals given the probability of reversals; optionally, the mass is plotted.
mass.B <- function(p,plot=FALSE)
{	
	p    <- p[p>0];													# Discard reversals that occur with zero probability.
	n    <- length(p)+1;											# Determine the number of possible reversals.
	dist <- rep(0,n); 												# Create space to store the distribution.
	dist[1] <- 1;													# Initialize the distribution with the degenerate probability 1.
	for (i in 1:(n-1)) { dist <- (1-p[i])*dist+p[i]*c(0,dist[-n]) }	# Update the distribution for each new probability.
	if (plot)														# If plot is selected, plot the probability mass function.
	{
		plot(0:n,dist,type="h",ylab="Probability",xlab="Number of Reversals",main="Probability Mass Function");
		points(0:n,dist,pch=16);
	}
	return(as.data.frame(dist,row.names=as.character(0:(n-1))));	# Return distribution as data frame of probabilities; can check sum(dist)==1.
}

# Calculate the p-values for reversals and optionally plots the relevant probability mass function(s).
# Type can be "S" for strong, "s" for standard, "n" for nonstandard, or some combination thereof.
mass.A <- function(m=NA,row=1:5,plot=FALSE,type=c("s","n"),cond=FALSE,prob=TRUE,pv=FALSE,...) # mass.A("CE",1:2,type="s")
{
	# Declare function that pulls a given row (point) from reversal observations, when the index is an even multiple of the row.
	getRow <- function(y,row,mean=TRUE)
	{
		if (all(row==1:5)) { return(y); }			# If all rows are requested, return the data unchanged.
		row <- c(5:1)[row]							# Reverse row order so that 1 is "closest" and 5 "furthest" from the boundary.
		row <- row%%5;								# Take the desired row and modulus by five to get appropriate row.
		y <- y[as.integer(row.names(y))%%5==row,];	# Grab the appropriate row.
		return(y);									# Return the mean or full subset of the desired row.
	}	

	# Run mle.A and extract the reversal probabilities for the correct row.
	if (any(is.na(m))) { y <- getRow(reversal.A(reversal=TRUE,silent=TRUE,strong=all(type=="S"),...),row); }
	else { m <- getRow(m,row); }
	
	# Create space to store realizations and probabilities of reversals.
	r.CE <- c(); p.CE <- c();
	r.PE <- c(); p.PE <- c();
	r.PL <- c(); p.PL <- c();
	r.PR <- c(); p.PR <- c();
	
	if (any(type=="n")) # Extract nonstandard reversals.
	{
		# Extract realizations of standard reversals.
		if (cond)
		{
			r.CE <- c(r.CE,m$CE.rn);
			r.PR <- c(r.PR,m$PR.rn);
			r.PE <- c(r.PE,m$PE.rn);	
			r.PL <- c(r.PL,m$PL.rn);
		}
		else
		{
			r.CE <- c(r.CE,m$CE.rn);
			r.PR <- c(r.PR,m$PR.rn);
			r.PE <- c(r.PE,m$PE.rn);	
			r.PL <- c(r.PL,m$PL.rn);
		}

		# Extract probabilities of standard reversals.
		if (cond)
		{
			p.CE <- c(p.CE,m$CE.pn);
			p.PR <- c(p.PR,m$PR.pn);
			p.PE <- c(p.PE,m$PE.pn);
			p.PL <- c(p.PL,m$PL.pn);
		}
		else
		{
			p.CE <- c(p.CE,m$CE.pn);
			p.PR <- c(p.PR,m$PR.pn);
			p.PE <- c(p.PE,m$PE.pn);
			p.PL <- c(p.PL,m$PL.pn);
		}
	}
	if (any(type=="s")) # Extract standard reversals.
	{
		# Extract realizations of standard reversals with non-zero probability.
		if (cond)
		{
			r.CE <- c(r.CE,m$CE.rs);
			r.PR <- c(r.PR,m$PR.rs);
			r.PE <- c(r.PE,m$PE.rs);	
			r.PL <- c(r.PL,m$PL.rs);
		}
		else
		{
			r.CE <- c(r.CE,m$CE.rs);
			r.PR <- c(r.PR,m$PR.rs);
			r.PE <- c(r.PE,m$PE.rs);	
			r.PL <- c(r.PL,m$PL.rs);
		}
	
		# Extract non-zero probabilities of standard reversals.
		if (cond)
		{
			p.CE <- c(p.CE,m$CE.ps);
			p.PR <- c(p.PR,m$PR.ps);
			p.PE <- c(p.PE,m$PE.ps);
			p.PL <- c(p.PL,m$PL.ps);
		}
		else
		{
			p.CE <- c(p.CE,m$CE.ps);
			p.PR <- c(p.PR,m$PR.ps);
			p.PE <- c(p.PE,m$PE.ps);
			p.PL <- c(p.PL,m$PL.ps);
		}
	}
	else if (any(type=="S")) # Extract strong reversals (which only occur for CE and PR).
	{
		# Extract realizations of standard reversals with non-zero probability.
		if (cond)
		{
			r.CE <- c(r.CE,m$CE.rS[m$CE.pS>0]);
			r.PR <- c(r.PR,m$PR.rS[m$PR.pS>0]);	
		}
		else
		{
			r.CE <- c(r.CE,m$CE.rS);
			r.PR <- c(r.PR,m$PR.rS);	
		}	
		
		# Extract non-zero probabilities of standard reversals.
		if (cond)
		{
			p.CE <- c(p.CE,m$CE.pS[m$CE.pS>0]);
			p.PR <- c(p.PR,m$PR.pS[m$PR.pS>0]);
		}
		else
		{
			p.CE <- c(p.CE,m$CE.pS);
			p.PR <- c(p.PR,m$PR.pS);			
		}
		
	}
	
	if (pv|plot)
	{
		# Calculate probability masses for all types.
		p.CE <- mass.B(p.CE);
		p.PE <- mass.B(p.PE);
		p.PL <- mass.B(p.PL);
		p.PR <- mass.B(p.PR);
	
		# Calculate equal-tail p-values. 
		p    <- data.frame(row.names="p.value")
		p$CE <- 2*pmin(sum(p.CE$dist[1:sum(r.CE)]),1-sum(p.CE$dist[1:sum(r.CE)]))
		p$PE <- 2*pmin(sum(p.PE$dist[1:sum(r.PE)]),1-sum(p.PE$dist[1:sum(r.PE)]))
		p$PL <- 2*pmin(sum(p.PL$dist[1:sum(r.PL)]),1-sum(p.PL$dist[1:sum(r.PL)]))
		p$PR <- 2*pmin(sum(p.PR$dist[1:sum(r.PR)]),1-sum(p.PR$dist[1:sum(r.PR)]))
		p[is.na(p)] <- 1
	}
	else if (!any(type=="s")&!any(type=="n"))
	{
	 	p <- data.frame(row.names="reversal")
	 	if (!prob)
	 	{
	 		p$CE <- mean(r.CE,na.rm=TRUE);
	 		p$PR <- mean(r.PR,na.rm=TRUE);
	 	}
	 	else
	 	{
	 		p$CE <- mean(p.CE,na.rm=TRUE);
	 		p$PR <- mean(p.PR,na.rm=TRUE);
	 	}
	 	p[is.na(p)] <- 1
	}
	else
	{
		p <- data.frame(row.names="reversal")
		if (!prob)
		{
			p$CE <- mean(r.CE,na.rm=TRUE);
			p$PE <- mean(r.PE,na.rm=TRUE);
			p$PL <- mean(r.PL,na.rm=TRUE);
			p$PR <- mean(r.PR,na.rm=TRUE);
		}
		else
		{
			p$CE <- mean(p.CE,na.rm=TRUE);
			p$PE <- mean(p.PE,na.rm=TRUE);
			p$PL <- mean(p.PL,na.rm=TRUE);
			p$PR <- mean(p.PR,na.rm=TRUE);
		}
		p[is.na(p)] <- 1
	}
	
	# Plot the probability mass function.
	if(plot&!all(type=="S"))
	{
		# Prepare a 2x2 plot for graphing. If the number of lines to be plotted is sufficiently sparse, also print dots.
		par(mfrow=c(2,2),oma=c(0,0,2,0));
		sparse <- (max(length(p.CE$dist),length(p.PE$dist),length(p.PL$dist),length(p.PR$dist))>50)
		
		# Plot probability mass function for CE.
		xlab = paste("Number of Reversals (",sum(r.CE)," Observed)",sep="");
		plot(c(0:(nrow(p.CE)-1)),p.CE$dist,type="h",ylab="Probability",xlab=xlab,main="Certainty Equivalence");
		if (sparse) { points(c(0:(nrow(p.CE)-1)),p.CE$dist,pch=16); }
		points(sum(r.CE),0,col="red",pch=17);
		
		# Plot probability mass function for PE.
		xlab = paste("Number of Reversals (",sum(r.PE)," Observed)",sep="");
		plot(c(0:(nrow(p.PE)-1)),p.PE$dist,type="h",ylab="Probability",xlab=xlab,main="Probability Equivalence");
		if (sparse) { points(c(0:(nrow(p.PE)-1)),p.PE$dist,pch=16); }
		points(sum(r.PE),0,col="red",pch=17);
		
	#	# Plot probability mass function for PL.
	#	xlab = paste("Number of Reversals (",sum(r.PL)," Observed)",sep="");
	#	plot(c(0:(nrow(p.PL)-1)),p.PL$dist,type="h",ylab="Probability",xlab=xlab,main="Project Left");
	#	if (sparse) { points(c(0:(nrow(p.PL)-1)),p.PL$dist,pch=16); }
	#	points(sum(r.PL),0,col="red",pch=17);
		
		# Plot probability mass function for PR.
		xlab = paste("Number of Reversals (",sum(r.PR)," Observed)",sep="");
		plot(c(0:(nrow(p.PR)-1)),p.PR$dist,type="h",ylab="Probability",xlab=xlab,main="Project Right");
		if (sparse) { points(c(0:(nrow(p.PR)-1)),p.PR$dist,pch=16); }
		points(sum(r.PR),0,col="red",pch=17);
	}
	else if (plot&all(type=="S"))
	{
		# Prepare a 2x2 plot for graphing. If the number of lines to be plotted is sufficiently sparse, also print dots.
		par(mfrow=c(1,2),oma=c(0,0,2,0));
		sparse <- (max(length(p.CE$dist),length(p.PR$dist))>50)
		
		# Plot probability mass function for CE.
		xlab = paste("Number of Reversals (",sum(r.CE)," Observed)",sep="");
		plot(c(0:(nrow(p.CE)-1)),p.CE$dist,type="h",ylab="Probability",xlab=xlab,main="Certainty Equivalence");
		if (sparse) { points(c(0:(nrow(p.CE)-1)),p.CE$dist,pch=16); }
		points(sum(r.CE),0,col="red",pch=17);
		
		# Plot probability mass function for PR.
		xlab = paste("Number of Reversals (",sum(r.PR)," Observed)",sep="");
		plot(c(0:(nrow(p.PR)-1)),p.PR$dist,type="h",ylab="Probability",xlab=xlab,main="Project Right");
		if (sparse) { points(c(0:(nrow(p.PR)-1)),p.PR$dist,pch=16); }
		points(sum(r.PR),0,col="red",pch=17);
	}
	# Otherwise return probability mass function and realizations as a list.
	# else { return(list(p.CE,p.PE,p.PL,p.PR,r.CE,r.PE,r.PL,r.PR)); }
	# Otherwise return equal-tail p-values.
	else { return(p); }
#	if (prob) { return(p); }
#	else      { return(colMeans(cbind(r.CE,r.PE,r.PL,r.PR))); }
#	}
}

# Note that rounding error may make for very small negative p-values.
mass.R <- function(s=1:60,prob=FALSE,type=FALSE,cons=FALSE,strong=FALSE,pv=FALSE)
{
	x <- reversal.A(s=s,prob=prob,type=type,cons=cons,strong=strong);										# Get reversal numbers and probabilities.
	y <- as.data.frame(matrix(nrow=6,ncol=ifelse(strong,2,8)));												# Create a data frame to store p-values.
	if (strong) { colnames(y) <- c("CE.S","PR.S"); }														# Create columns for strong reversals.
	else { colnames(y) <- c("CE.s","CE.n","PE.s","PE.n","PL.s","PL.n","PR.s","PR.n"); }						# Create columns for relevant reversal numbers.
	
	if (cons) { rows <- c(2,4); } else { rows <- 1:5; }														# Set the number of rows for consistency checks.
 	for (i in rows)																							# Iterate through all five points on both sides.
	{
		if (strong) { y[i,c("CE.S","PR.S")] <- mass.A(x,row=i,type="S",prob=prob,pv=pv)[,c("CE","PR")]; }	# Calculate the p-values for strong reversals.
		else
		{
			y[i,c("CE.s","PE.s","PL.s","PR.s")] <- mass.A(x,row=i,type="s",prob=prob,pv=pv);				# Calculate the p-values for standard reversals.
			y[i,c("CE.n","PE.n","PL.n","PR.n")] <- mass.A(x,row=i,type="n",prob=prob,pv=pv);				# Calculate the p-values for nonstandard reversals.
		}
	}
	if (!cons)
	{
		if (strong) { y[6,c("CE.S","PR.S")] <- mass.A(x,row=1:5,type="S",prob=prob,pv=pv)[,c("CE","PR")]; }	# Calculate the p-values for all strong reversals
		else
		{
			y[6,c("CE.s","PE.s","PL.s","PR.s")] <- mass.A(x,row=1:5,type="s",prob=prob,pv=pv);				# Calculate the p-values for all standard reversals.
			y[6,c("CE.n","PE.n","PL.n","PR.n")] <- mass.A(x,row=1:5,type="n",prob=prob,pv=pv);				# Calculate the p-values for all nonstandard reversals.
		}
	}
	
	if (cons)
	{
		y <- y[c(rows),];
		rownames(y) <- rows;
	}
	return(y);																								# Return the calculated p-values.
}

# Calculate the p-values for all monotonicity violations.
# The argument bet is "p" for p-bet, "d" for dollar bet, and c("p","d") for both.
mass.M <- function(bet=c("p","d"))
{
	# Either load or the monotonicity probability estimates from a file or estimate them.
	# load("mono.Rdata");
	y <- mle.A(mono=TRUE);
	#save(y,file="mono.Rdata");

	# Subset the data by p-bet or $-bet. Obviously the function can be run twice if 
	# (Keep in mind that there are ten monotonicity pairs and they rotate between p-bet and $-bet.)
	if (all(bet=="p")) { y[[2]] <- subset(y[[2]],as.integer(row.names(y[[2]]))%%20<=10&as.integer(row.names(y[[2]]))%%20!=0) }	# Grab only p-bets.
	if (all(bet=="d")) { y[[2]] <- subset(y[[2]],as.integer(row.names(y[[2]]))%%20>10|as.integer(row.names(y[[2]]))%%20==0)  }	# Grab only $-bets.
	
	# Declare a function to pull data from a particular of gambles by treatment.
	getSubset <- function(y,t,dominated,dominant)
	{
		if      (t=="CE")															# If the treatment is CE, grab relevant data.
		{
			obs  <- subset(y$CE.mv,y$dominated==dominated&y$dominant==dominant);	# Grab observed monotonicity violations for CE.
			prob <- subset(y$CE.pv,y$dominated==dominated&y$dominant==dominant);	# Grab estimated probabilities of a violation for CE.
		}
		else if (t=="PE")
		{
			obs  <- subset(y$PE.mv,y$dominated==dominated&y$dominant==dominant);	# Grab observed monotonicity violations for PE.
			prob <- subset(y$PE.pv,y$dominated==dominated&y$dominant==dominant);	# Grab estimated probabilities of a violation for PE.
		}
		else if (t=="PR")
		{
			obs  <- subset(y$PR.mv,y$dominated==dominated&y$dominant==dominant);	# Grab observed monotonicity violations for PR.
			prob <- subset(y$PR.pv,y$dominated==dominated&y$dominant==dominant);	# Grab estimated probabilities of a violation for PR.
		}
		return(as.data.frame(cbind(obs,prob)));										# Return subset of observed violations and associated probs.
	}

	# Calculate equal-tail p-values. 
	p.value <- function(y)
	{
		dist <- mass.B(y$prob)$dist;
		return(2*pmin(sum(dist[1:sum(y$obs)]),sum(dist[(1+sum(y$obs)):length(dist)])));
	}

	# Calculate p-values for each pair of dominated and dominant gamble.
	p <- data.frame(matrix(nrow=0,ncol=5));											# Create a data frame to store p-values.
	for (i in 1:4)																	# Iterate through dominated gambles.
	{
		for (j in 5:(i+1))															# Iterate through dominant gambles (in reversed order).
		{
			p.CE <- getSubset(y[[2]],"CE",i,j);										# Get the appropriate pair of gambles for CE.
			p.PE <- getSubset(y[[2]],"PE",i,j);										# Get the appropriate pair of gambles for PE.
			p.PR <- getSubset(y[[2]],"PR",i,j);										# Get the appropriate pair of gambles for PR.
			p <- rbind(p,cbind(i,c(5:1)[j],mean(p.CE$obs),mean(p.PE$obs),mean(p.PR$obs),mean(p.CE$prob),mean(p.PE$prob),mean(p.PR$prob),p.value(p.CE),p.value(p.PE),p.value(p.PR)));
		}
	}
	
	# Name the columns for output.
	colnames(p) <- c("dominated","dominant","CE.mv","PE.mv","PR.mv","CE.mp","PE.mp","PR.mp","CE.pv","PE.pv","PR.pv");

	# Return the observed monotonicity violations and p-values.
	return(p);
}

######################################################################################################################
# Declare functions for plotting parameters and the PDF and CDF of the probability and certainty equivalents.		 #
######################################################################################################################

# Calculates the Blavasktykyy confidence interval critical values of a lottery A (omitting A or pA) for some alternative lottery B.
interval.B <- function(A=NA,B=NA,pA=NA,pB=NA,a=NA,r=NA,pAmin=0,pAmax=1,CI=.95,N=100)
{
	# Find confidence interval for cash equivalents (or certainty equivalents, for degenerate alternatives.)
	if (is.na(A)) 
	{
		# Declare functions for numerical derivation of exact confidence interval critical values.
	  	CE.lower <- function(X,B,pA,pB,a,r,CI) { return(abs(F(X,B,pA,pB,a,r)-((1-CI)/2))); }
		CE.upper <- function(X,B,pA,pB,a,r,CI) { return(abs(F(X,B,pA,pB,a,r)-((1+CI)/2))); }

		# Do a coarse grid search for starting critical values.
		x <- seq(0,B,length.out=N);
		y <- rep(0,N);
		for (i in 1:N) { y[i] <- F(x[i],B,pA,pB,a,r); }
		
		# Find the upper and lower bounds of the interval (e.g. the critical values) using an optimization algorithm.
		lower <- optim(x[which.min(abs(y-CI))],method="L-BFGS-B",lower=0,upper=B,fn=CE.lower,B=B,pA=pA,pB=pB,a=a,r=r,CI=CI)$par
		upper <- optim(x[which.min(abs(y-(1-CI)))],method="L-BFGS-B",lower=0,upper=B,fn=CE.upper,B=B,pA=pA,pB=pB,a=a,r=r,CI=CI)$par
	}
	# Find confidence interval for probability equivalents (within [pAmin,pAmax] as appropriate for PL and PR).
	else if (is.na(pA))
	{
		# Declare functions for numerical derivation of exact confidence interval critical values.
  		PE.lower <- function(X,A,B,pB,a,r,CI) { return(abs(F(A,B,X,pB,a,r)-((1-CI)/2))); }
		PE.upper <- function(X,A,B,pB,a,r,CI) { return(abs(F(A,B,X,pB,a,r)-((1+CI)/2))); }
		
		# Do a coarse grid search for starting critical values.
		x <- seq(pAmin,pAmax,length.out=N);
		y <- rep(0,N);
		for (i in 1:N) { y[i] <- F(A,B,x[i],pB,a,r); }
		
		# Find the upper and lower bounds of the interval (e.g. the critical values) using an optimization algorithm.
		lower <- optim(x[which.min(abs(y-CI))],method="L-BFGS-B",lower=pAmin,upper=pAmax,fn=PE.lower,A=A,B=B,pB=pB,a=a,r=r,CI=CI)$par
		upper <- optim(x[which.min(abs(y-(1-CI)))],method="L-BFGS-B",lower=pAmin,upper=pAmax,fn=PE.upper,A=A,B=B,pB=pB,a=a,r=r,CI=CI)$par
	}	
	
	# Return the critical values of the confidence interval.
	return(c(lower,upper))
}

# Plot a Butler & Loomes (2007) "style" graph using Blavatskyy confidence intervals.
# Figure 1: plot.CI("");
# Figure 2: plot.CI("CE",i=2,j=5);
# Figure 3: plot.CI();
plot.CI <- function(t=c("CE","PE","PL","PR"),i=2,j=NA,a=NA,r=NA,CI=.35,N=100,silent=FALSE)
{
	p <- param.B()[,1:4];															# Extract vector of relevant parameters.
	h <- c(1:22)[-c(7,17,21,22)]													# Create vector of redundant & degenerate points to not graph.
	L <- paste(c(rep("P",9),rep("D",9)),as.character(c(5:1,6:9,5:1,6:9)),sep="");	# Create labels for P-bets and $-bets.

	if (is.na(a)|is.na(r))															# If either alpha or rho is undeclared, assign them.
	{
		if (is.na(a)&is.na(r))														# If neither alpha or rho is declared, select "typical subject".
		{
			y <- mle.A();															# Estimate alpha and rho for all subjects.
			y.median <- c(median(y$alpha),median(y$rho));							# Find the median alpha and rho.
			y.dist <- vector();														# Create a vector to store Euclidian distance from median {alpha,rho}.
			for (k in 1:60) { y.dist[k] <- dist(rbind(c(y$alpha[k],y$rho[k]),y.median))[1] }
			a <- y$alpha[which.min(y.dist)];										# Find alpha for subject that minimized Euclidian distance.
			r <- y$rho[which.min(y.dist)];											# Find rho for subject that minimized Euclidian distance.
			if (!silent) { print(paste("Subject ",which.min(y.dist)," is selected: α≈",round(a,4)," and ρ≈",round(r,4),".",sep="")); }	
			# y <- mle.B(fetch.B(1:60,t=c("CE","PE","BC")))[1:2]; a<-y[1]; r<-y[2]; # Use "population" parameters instead.
		}
		else if (is.na(a)) { a <- 1; }												# If no alpha only, use unitary alpha.
		else if (is.na(r)) { r <- 0; }												# If no rho only, use zero (risk neutral.)
	}
	
	# If silent plot is off, print probabilities of binary choice(s).
	if (i>0&!silent)
	{
		prob <- F(A=p$HoutcomeA[i],B=p$HoutcomeA[i+10],pA=p$HprobA[i]/100,pB=p$HprobA[i+10]/100,a=a,r=r);	# Calculate P(A,B) for point i.
		print(paste("The probability that p-bet is selected over $-bet is approximately ",round(prob,4),".",sep=""));
	}
	if (!is.na(j)&!silent)
	{
		prob <- F(A=p$HoutcomeA[j],B=p$HoutcomeA[j+10],pA=p$HprobA[j]/100,pB=p$HprobA[j+10]/100,a=a,r=r);	# Calculate P(A,B) for point j.
		print(paste("For the second binary choice, the probability is approximately ",round(prob,4),".",sep=""));
	}
	
	dev.new(width=5.5,height=4.125);												# Create a new plot window.
	par(mar=c(2.5, 3, 1.5, 0.5),mgp=c(3, .5, 0));									# Eliminate excess margins.
	if (i>1&length(t)>1)
	{
		par(mfrow=c(2,2));															# If there is more than one treatment to plot, make four boxes.
		pinc <- 20;																	# Reduce the increment by which percentages are generated on the axis.
		dots <- .5;																	# Reduce the size of the dots at which the points are plotted.
	}
	else
	{
		pinc <- 10;																	# Reduce the increment by which percentages are generated on the axis.
		dots <- .75;																# Increase the size of the dots at which the points are plotted.
	}				

	# Iterate through treatments and plot graphs.
	for (k in 1:length(t))
	{
		# Prepare graph title (for use with multiple graphs.)
		if      (t[k]=="CE") { main="Project Up (Cash for Lottery)"; }
		else if (t[k]=="PE") { main="Project Down (Lottery for Cash)"; }	
		else if (t[k]=="PL") { main="Project Left (Lottery for Lottery)"; }
		else if (t[k]=="PR") { main="Project Right (Lottery for Lottery)"; }
		
		# Prepare graph title and subtitle (for use with multiple graphs.)
		# if      (t[k]=="CE") { main="Project Up"; subtitle="Certainty Equivalent, Cash for Lottery";	}
		# else if (t[k]=="PE") { main="Project Down"; subtitle="Probability Equivalent, Lottery for Cash"; }	
		# else if (t[k]=="PL") { main="Project Left"; subtitle="Probability Equivalent, Lottery for Lottery"; }
		# else if (t[k]=="PR") { main="Project Right"; subtitle="Probability Equivalent, Lottery for Lottery"; }
	
		# Plot the relevant points, label them, draw the axes, and label the y-axis.
		plot(x=p$HoutcomeA[h],y=p$HprobA[h]/100,xlim=c(0,80),ylim=c(0,1),pch=20,xaxs="i",yaxs="i",xaxt="n",yaxt="n",ann=FALSE,family="Times",cex=dots)
		text(x=p$HoutcomeA[h]+3,y=p$HprobA[h]/100,labels=L,cex=.7,family="Times")
		axis(side=2,at=seq(0,1,pinc/100),labels=paste(as.character(seq(0,100,pinc)),"%",sep=""),las=1,cex.axis=.8,family="Times")
		axis(side=1,at=seq(0,80,10),labels=paste("$",as.character(seq(0,80,10)),sep=""),cex.axis=.8,family="Times")
		if (length(t)>1) { title(main=main,family="Times",cex.main=1) }											# Print a title for multiple graphs.
		# if (length(t)>1) { title(main=main,sub=subtitle,family="Times",cex.main=1) }							# Print a title & subtitle for multiple graphs.
		title(ylab="Probability of Up-state Payoff",family="Times",cex.lab=.8,line=2)							# Print only the y-axis title for now.
		title(xlab="Up-state Lottery Payoff (with $0 Down-state)",family="Times",cex.lab=.8,line=(.75+dots))	# Print the x-axis

		# Draw lines across at the Bulter and Loomes (2007) points.
		abline(v=p$HoutcomeA[2], lty=2,lwd=.5);																	# Vertical line for p-bet.
		abline(v=p$HoutcomeA[12], lty=2,lwd=.5);																# Vertical line for $-bet.
		abline(h=p$HprobA[2]/100, lty=2,lwd=.5);																# Horizontal line for p-bet.
		abline(h=p$HprobA[12]/100, lty=2,lwd=.5);																# Horizontal line for $-bet.
	
		# Calculate the point from which the cone emanates and the Left/Top and Right/Bottom points.
		if (i>0&t[k]=="CE")
		{	
			# Plot lower and upper bound of confidence interval for CE p-bet.
			y <- seq(p$HprobA[i]/100,1,.1/N)[-1]																# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(y),2) 																# Create space to store interval critical values.
			for (l in 1:length(y)) { ci[l,1:2] <- interval.B(A=NA,B=p$HoutcomeA[i],pA=y[l],pB=p$HprobA[i]/100,a=a,r=r,CI=CI); }
			lines(x=ci[,1],y=y)																					# Plot lower bound of interval.
			lines(x=ci[,2],y=y)																					# Plot upper bound of interval.
			
			# Plot lower and upper bound of confidence interval for CE $-bet.
			y <- seq(p$HprobA[i+10]/100,1,.1/N)[-1]
			ci <- matrix(numeric(0),length(y),2) 																# Create space to store interval critical values. 
			for (l in 1:length(y)) { ci[l,1:2] <- interval.B(A=NA,B=p$HoutcomeA[i+10],pA=y[l],pB=p$HprobA[i+10]/100,a=a,r=r,CI=CI); }
			lines(x=ci[,1],y=y)																					# Plot lower bound of interval.
			lines(x=ci[,2],y=y)																					# Plot upper bound of interval.
		}
		else if (i>0&t[k]=="PE")
		{
			# Plot lower and upper bound of confidence interval for PE p-bet.
			x <- seq(p$Hupfront[i],p$HoutcomeA[i],1/N)[-1]														# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(x),2) 																# Create space to store interval critical values. 
			for (l in 1:length(x)) { ci[l,1:2] <- interval.B(A=x[l],B=p$Hupfront[i],pA=NA,pB=1,a=a,r=r,CI=CI); }
			lines(x=x,y=ci[,1])																					# Plot lower bound of interval.
			lines(x=x,y=ci[,2])																					# Plot upper bound of interval.
			
			# Plot lower and upper bound of confidence interval for PE $-bet.
			x <- seq(p$Hupfront[i+10],p$HoutcomeA[i+10],1/N)[-1]												# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(x),2)  																# Create space to store interval critical values.
			for (l in 1:length(x)) { ci[l,1:2] <- interval.B(A=x[l],B=p$Hupfront[i+10],pA=NA,pB=1,a=a,r=r,CI=CI); }
			lines(x=x,y=ci[,1])																					# Plot lower bound of interval.
			lines(x=x,y=ci[,2])																					# Plot upper bound of interval.
		}
		else if (i>0&t[k]=="PL")
		{
			# Plot lower and upper bound of confidence interval for PL p-bet.
			x <- seq(p$HoutcomeA[i+5],80,1/N)[-1]																# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(x),2)  																# Create space to store interval critical values.
			for (l in 1:length(x)) { ci[l,1:2] <- interval.B(A=x[l],B=80,pA=NA,pB=p$Hrefprob[i+5]/100,a=a,r=r,pAmin=p$Hrefprob[i+5]/100,CI=CI); }
			lines(x=x,y=ci[,1])																					# Plot lower bound of interval.
			lines(x=x,y=ci[,2])																					# Plot upper bound of interval.
			
			# Plot lower and upper bound of confidence interval for PL $-bet.
			x <- seq(p$HoutcomeA[i+15],80,1/N)[-1]																# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(x),2)  																# Create space to store interval critical values.
			for (l in 1:length(x)) { ci[l,1:2] <- interval.B(A=x[l],B=80,pA=NA,pB=p$Hrefprob[i+15]/100,a=a,r=r,pAmin=p$Hrefprob[i+15]/100,CI=CI); }
			lines(x=x,y=ci[,1])																					# Plot lower bound of interval.
			lines(x=x,y=ci[,2])																					# Plot upper bound of interval.
		}
		else if (i>0&t[k]=="PR")
		{
			# Plot lower and upper bound of confidence interval for PR p-bet.
			x <- seq(p$HoutcomeA[i+5],80,1/N)[-1]																# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(x),2)  																# Create space to store interval critical values. 
			for (l in 1:length(x)) { ci[l,1:2] <- interval.B(A=x[l],B=p$HoutcomeA[i+5],pA=NA,pB=p$HprobA[i+5]/100,a=a,r=r,pAmax=p$HprobA[i+5]/100,CI=CI); }
			lines(x=x,y=ci[,1])																					# Plot lower bound of interval.
			lines(x=x,y=ci[,2])																					# Plot upper bound of interval.

			# Plot lower and upper bound of confidence interval for PR $-bet.
			x <- seq(p$HoutcomeA[i+15],80,1/N)[-1]																# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(x),2)   																# Create space to store interval critical values.
			for (l in 1:length(x)) { ci[l,1:2] <- interval.B(A=x[l],B=p$HoutcomeA[i+15],pA=NA,pB=p$HprobA[i+15]/100,a=a,r=r,pAmax=p$HprobA[i+15]/100,CI=CI); }
			lines(x=x,y=ci[,1])																					# Plot lower bound of interval.
			lines(x=x,y=ci[,2])																					# Plot upper bound of interval.
		}
		if (!is.na(j)&length(t)==1&any(t=="CE"))
		{
			# If a second plot is requested for CE, plot lower and upper bound of p-bet confidence interval.
			y <- seq(p$HprobA[j]/100,1,.1/N)[-1]																# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(y),2)    															# Create space to store interval critical values.
			for (l in 1:length(y)) { ci[l,1:2] <- interval.B(A=NA,B=p$HoutcomeA[j],pA=y[l],pB=p$HprobA[j]/100,a=a,r=r,CI=CI); }
			lines(x=ci[,1],y=y)																					# Plot lower bound of interval.
			lines(x=ci[,2],y=y)																					# Plot upper bound of interval.

			# If a second plot is requested for CE, plot lower and upper bound of $-bet confidence interval.
			y <- seq(p$HprobA[j+10]/100,1,.1/N)[-1]																# Create sequence to calculate interval bounds for.
			ci <- matrix(numeric(0),length(y),2)    															# Create space to store interval critical values.
			for (l in 1:length(y)) { ci[l,1:2] <- interval.B(A=NA,B=p$HoutcomeA[j+10],pA=y[l],pB=p$HprobA[j+10]/100,a=a,r=r,CI=CI); }
			lines(x=ci[,1],y=y)																					# Plot lower bound of interval.
			lines(x=ci[,2],y=y)																					# Plot upper bound of interval.
		}
	}
}

# Plot the CE and PE CDF and PDF of any gamble (with index i) for some alpha and rho.
plot.B <- function(s=NA,i=2,a=NA,r=NA,N=100,numDeriv=FALSE)
{
	# Load numeric differentiation library if required.
	if(numDeriv) { require(numDeriv); }

	# Load data for selected subject or the risk neutral agent.
	if(is.na(s))   { x <- sample.B(NA);a=1;r=0; }
	else
	{
		# Load data for selected subject.
		x <- disp.B(s);
		
		# Get estimated alpha and rho for subject if not specified.
		if(is.na(a)|is.na(r))
		{
			y <- mle.B(fetch.B(s,t=c("CE","PE")));
			if (is.na(a)) { a = y[1]; }
			if (is.na(r)) { r = y[2]; }
		}
	}

	# If the lottery has a PE/CE index, display the probability and certainty equivalents.
	if ((i>=1&i<=5)|(i>=11&i<=15))
	{
		# Find the CDF and PDF of the certainty equivalent.
		X  <- seq(1e-4,x$HoutcomeA[i]-1e-4,len=N+1);
		pX <- rep(1,N+1); # pX <- seq(0,1,len=N+1);
		
		CE.F <- F(X,rep(x$HoutcomeA[i],N+1),pX,rep(x$HprobA[i]/100,N+1),a,r);
		CE.f <- f(X,rep(x$HoutcomeA[i],N+1),pX,rep(x$HprobA[i]/100,N+1),a,r,filter=TRUE);

		# Find the CDF and PDF of the probability equivalent.
		Q  <- rep(x$HoutcomeA[i],N+1);
		pq <- seq(1e-4,1-1e-4,len=N+1); # pq <- seq(0,1,len=N+1);
		
		PE.F <- F(Q,rep(x$Hupfront[i],N+1),pq,rep(1,N+1),a,r);
		PE.f <- f(Q,rep(x$Hupfront[i],N+1),pq,rep(1,N+1),a,r,filter=TRUE);

		# Prepare a 2x2 plot for graphing.
		par(mfrow=c(2,2),oma=c(0,0,2,0));

		# Plot the CDF of the certainty equivalent.
		plot(X,CE.F,xlab="Certainty Equivalent",ylab="Cumulative Distribution Function",type="l");
		title(paste("Subject ",s," input CE=$",round(x$HCE[i],digits=2)," with P(CE<=",x$HCE[i],")=",round(F(x$HCE[i],x$HoutcomeA[i],1,x$HprobA[i]/100,a,r),digits=2),".",sep=""),cex.main=.75);
		abline(v=x$HCE[i], lty=1, col="red");
		abline(v=x$Hupfront[i], lty=1, col="grey90", lwd=.5);

		# Plot the CDF of the probability equivalent.
		plot(pq,PE.F,xlab="Probability Equivalent",ylab="Cumulative Distribution Function",type="l");
		title(paste("Subject ",s," input PE=",x$HPE[i]/100," with P(PE<=",x$HPE[i]/100,")=",round(F(x$HoutcomeA[i],x$Hupfront[i],x$HPE[i]/100,1,a,r),digits=2),".",sep=""),cex.main=.75);
		abline(v=x$HPE[i]/100, lty=1, col="red");
		abline(v=x$HprobA[i]/100, lty=1, col="grey90", lwd=.5);
	
		# Plot the PDF of the certainty equivalent.
		plot(X,CE.f,xlab="Certainty Equivalent",ylab="Probability Density Function",type="l");
		if (numDeriv) { points(X,f.xn(X,rep(x$HoutcomeA[i],N+1),pX,rep(x$HprobA[i]/100,N+1),a,r),type='l',col='green'); }
		title(paste("The density at P(CE=",x$HCE[i],")=",round(f(x$HCE[i],x$HoutcomeA[i],1,x$HprobA[i]/100,a,r,filter=TRUE),digits=2),".",sep=""),cex.main=.75);
		abline(v=x$HCE[i], lty=1, col="red"); # The probability of a particular response is 0; therefore this is not particularly interesting.
		abline(v=x$Hupfront[i], lty=1, col="grey90", lwd=.5);

		# Plot the PDF of the probability equivalent.
		plot(pq,PE.f,xlab="Probability Equivalent",ylab="Probability Density Function",type="l");
		if (numDeriv) { points(pq,f.pn(Q,rep(x$Hupfront[i],N+1),pq,rep(1,N+1),a,r),type='l',col='green'); }
		title(paste("The density at P(PE=",x$HPE[i]/100,")=",round(f(x$HoutcomeA[i],x$Hupfront[i],x$HPE[i]/100,1,a,r,filter=TRUE),digits=2),".",sep=""),cex.main=.75);
		abline(v=x$HPE[i]/100, lty=1, col="red"); # The probability of a particular response is 0; therefore this is not particularly interesting.
		abline(v=x$HprobA[i]/100, lty=1, col="grey90", lwd=.5);
		
		# Print the title on the graph.
		title(main = paste("CEs and PEs for Subject ",s," and Lottery ",i," ($",x$HoutcomeA[i]," with prob ",x$HprobA[i]/100," and EV=$",x$Hupfront[i],") with ","alpha=",round(a,digits=2)," and rho=",round(r,digits=2),".",sep=""), cex.main=.9, outer=TRUE,);
	}
	else if ((i>=6&i<=10)|(i>=16&i<=20))
	{
		# Find the CDF and PDF of the project left probability equivalent.
		PL.Q  <- rep(x$HoutcomeA[i],N+1);
		PL.pQ <- seq(1e-4,1-1e-4,len=N+1);
		PL.F  <- F(PL.Q,rep(80,N+1),PL.pQ,rep(x$HrefprobA[i]/100,N+1),a,r);
		PL.f  <- f(PL.Q,rep(80,N+1),PL.pQ,rep(x$HrefprobA[i]/100,N+1),a,r,filter=TRUE);
		# cbind(PL.Q,rep(80,N+1),PL.pQ,rep(x$HrefprobA[i]/100,N+1))

		# Find the CDF and PDF of the project right probability equivalent.
		PR.Q  <- rep(80,N+1);
		PR.pQ <- seq(1e-4,1-1e-4,len=N+1);
		PR.F  <- F(PR.Q,rep(x$HoutcomeA[i],N+1),PR.pQ,rep(x$HprobA[i]/100,N+1),a,r);
		PR.f  <- f(PR.Q,rep(x$HoutcomeA[i],N+1),PR.pQ,rep(x$HprobA[i]/100,N+1),a,r,filter=TRUE);
		# cbind(PR.Q,rep(x$HoutcomeA[i],N+1),PR.pQ,rep(x$HprobA[i]/100,N+1))

		# Prepare a 2x2 plot for graphing.
		par(mfrow=c(2,2),oma=c(0,0,2,0));
		
		# Plot the CDF of the project left probability equivalent.
		plot(PL.pQ,PL.F,xlab="Project Left Probability Equivalent",ylab="Cumulative Distribution Function",type="l");
		title(paste("Subject ",s," input PL:PE=",x$HPL[i]/100," with P(PE<=",x$HPL[i]/100,")=",round(F(x$HoutcomeA[i],80,x$HPL[i]/100,x$HrefprobA[i]/100,a,r),digits=2),".",sep=""),cex.main=.75);
		abline(v=x$HPL[i]/100, lty=1, col="red");
		abline(v=x$HprobA[i]/100, lty=1, col="grey90", lwd=.5);

		# Plot the CDF of the project right probability equivalent.
		plot(PR.pQ,PR.F,xlab="Project Right Probability Equivalent",ylab="Cumulative Distribution Function",type="l");
		title(paste("Subject ",s," input PR:PE=",x$HPR[i]/100," with P(PE<=",x$HPR[i]/100,")=",round(F(80,x$HoutcomeA[i],x$HPR[i]/100,x$HprobA[i]/100,a,r),digits=2),".",sep=""),cex.main=.75);
		abline(v=x$HPR[i]/100, lty=1, col="red");
		abline(v=x$HrefprobA[i]/100, lty=1, col="grey90", lwd=.5);

		# Plot the PDF of the project left probability equivalent.
		plot(PL.pQ,PL.f,xlab="Project Left Probability Equivalent",ylab="Probability Density Function",type="l");
		if (numDeriv) { points(PL.pQ,f.pn(PL.Q,rep(80,N+1),PL.pQ,rep(x$HrefprobA[i]/100,N+1),a,r),type='l',col='green'); }
		title(paste("The density at P(PL:PE=",x$HPL[i]/100,")=",round(f(x$HoutcomeA[i],80,x$HPL[i]/100,x$HrefprobA[i]/100,a,r,filter=TRUE),digits=2),".",sep=""),cex.main=.75);
		abline(v=x$HPL[i]/100, lty=1, col="red");
		abline(v=x$HprobA[i]/100, lty=1, col="grey90", lwd=.5);

		# Plot the PDF of the project right probability equivalent.
		plot(PR.pQ,PR.f,xlab="Project Right Probability Equivalent",ylab="Probability Density Function",type="l");
		if (numDeriv) { points(PR.pQ,f.pn(PR.Q,rep(x$HoutcomeA[i],N+1),PR.pQ,rep(x$HprobA[i]/100,N+1),a,r),type='l',col='green'); }
		title(paste("The density at P(PR:PE=",x$HPR[i]/100,")=",round(f(80,x$HoutcomeA[i],x$HPR[i]/100,x$HprobA[i]/100,a,r,filter=TRUE),digits=2),".",sep=""),cex.main=.75);
		abline(v=x$HPR[i]/100, lty=1, col="red");
		abline(v=x$HrefprobA[i]/100, lty=1, col="grey90", lwd=.5);

		# Print the title on the graph.
		title(main = paste("RL PEs for Subject ",s," and Lottery ",i," ($",x$HoutcomeA[i]," with prob ",x$HprobA[i]/100," and RP=",x$HrefprobA[i]/100," for $80) with ","alpha=",round(a,digits=2)," and rho=",round(r,digits=2),sep=""), cex.main=.9, outer=TRUE,);
	}
	# If the lottery index is out of bounds, report an error.
	else { warning(paste("Lottery",i,"is invalid. Please choose a lottery between 1 and 20 inclusive.")) }
}